@@ -157,7 +157,7 @@
          (emit-alignment sb-vm:n-lowtag-bits
                          ;; EMIT-LONG-NOP does not exist for (not x86-64)
                          #+x86-64 :long-nop))
-       (when *compile-print*
+       (when cl:*compile-print*
          (format *error-output* "~S assembled~%" ',name)))))
 
 (defun arg-or-res-spec (reg)
