@@ -65,7 +65,7 @@
 
   (/show0 "entering !COLD-INIT")
   (setf (symbol-function '%failed-aver) #'!cold-failed-aver)
-  (!cold-init-hash-table-methods) ; needed by MAKE-READTABLE
+  (!readtable-cold-init)
   (setq *readtable* (make-readtable)
         *print-length* 6 *print-level* 3)
   (setq *error-output* (!make-cold-stderr-stream)
@@ -74,10 +74,11 @@
   (/show "testing '/SHOW" *print-length* *print-level*) ; show anything
   (unless (!c-runtime-noinform-p)
     (write-string "COLD-INIT... "))
+  (!cold-init-hash-table-methods)
   ;; Establish **initial-handler-clusters**
   (show-and-call sb-kernel::!target-error-cold-init)
   ;; And now *CURRENT-THREAD* and *HANDLER-CLUSTERS*
-  (sb-thread::init-initial-thread)
+  (sb-thread::init-main-thread)
 
   ;; Assert that FBOUNDP doesn't choke when its answer is NIL.
   ;; It was fine if T because in that case the legality of the arg is certain.
@@ -173,12 +174,12 @@
         (function
          (funcall toplevel-thing))
         ((cons (eql :load-time-value))
-            (setf (svref *!load-time-values* (third toplevel-thing))
-                  (funcall (second toplevel-thing))))
+         (setf (svref *!load-time-values* (third toplevel-thing))
+               (funcall (second toplevel-thing))))
         ((cons (eql :load-time-value-fixup))
          (destructuring-bind (object index value) (cdr toplevel-thing)
            (aver (typep object 'code-component))
-           (aver (eq (code-header-ref object index) (make-unbound-marker)))
+           (aver (unbound-marker-p (code-header-ref object index)))
            (setf (code-header-ref object index) (svref *!load-time-values* value))))
         ((cons (eql defstruct))
          (apply 'sb-kernel::%defstruct (cdr toplevel-thing)))
@@ -257,8 +258,6 @@
       (logically-readonlyize (sb-c::sc-move-vops sc))
       (logically-readonlyize (sb-c::sc-move-costs sc))))
 
-  ; hppa heap is segmented, lisp and c uses a stub to call eachother
-  #+hpux (%primitive sb-vm::setup-return-from-lisp-stub)
   ;; The system is finally ready for GC.
   (/show0 "enabling GC")
   (setq *gc-inhibit* nil)
@@ -315,12 +314,12 @@ Consequences are unspecified if serious conditions occur during EXIT
 excepting errors from *EXIT-HOOKS*, which cause warnings and stop
 execution of the hook that signaled, but otherwise allow the exit
 process to continue normally."
-  (if (or abort *exit-in-process*)
+  (if (or abort *exit-in-progress*)
       (os-exit (or code 1) :abort t)
       (let ((code (or code 0)))
         (with-deadline (:seconds nil :override t)
           (sb-thread:grab-mutex *exit-lock*))
-        (setf *exit-in-process* code
+        (setf *exit-in-progress* code
               *exit-timeout* timeout)
         (throw '%end-of-the-world t)))
   (critically-unreachable "After trying to die in EXIT."))
@@ -331,12 +330,14 @@ process to continue normally."
   (sb-thread::init-job-control)
   (sb-thread::get-foreground))
 
-(defun reinit ()
+(defun reinit (total)
   ;; WITHOUT-GCING implies WITHOUT-INTERRUPTS.
   (without-gcing
     ;; Until *CURRENT-THREAD* has been set, nothing the slightest bit complicated
     ;; can be called, as pretty much anything can assume that it is set.
-    (sb-thread::init-initial-thread)
+    (when total ; newly started process, and not a failed save attempt
+      (sb-thread::init-main-thread)
+      #+sb-thread (setq *finalizer-thread* t))
     ;; Initializing the standard streams calls ALLOC-BUFFER which calls FINALIZE
     (finalizers-reinit)
     ;; Initialize streams next, so that any errors can be printed
