@@ -61,6 +61,9 @@
           ;; not that this is expected to work concurrently with gc.
           (make-room-info (ash most-positive-word (- (1+ n-widetag-bits)))
                           'bignum :other))
+    (setf (svref infos filler-widetag)
+          (make-room-info (ash most-positive-word (- (1+ n-widetag-bits)))
+                          'filler :other))
 
     (setf (svref infos closure-widetag)
           (make-room-info short-header-max-words 'closure :closure))
@@ -112,7 +115,7 @@
 (define-load-time-global *room-info* (!compute-room-infos))
 
 (defconstant-eqx +heap-spaces+
-  '((:dynamic   "Dynamic space"   sb-kernel:dynamic-usage)
+  '((:dynamic   "Dynamic space"   dynamic-usage)
     #+immobile-space
     (:immobile  "Immobile space"  sb-kernel::immobile-space-usage)
     (:read-only "Read-only space" sb-kernel::read-only-space-usage)
@@ -169,48 +172,55 @@
       (multiple-value-bind (start end) (%space-bounds space)
         (ash (- end start) n-fixnum-tag-bits))))
 
-;;; Round SIZE (in bytes) up to the next dualword boundary. A dualword
-;;; is eight bytes on platforms with 32-bit word size and 16 bytes on
-;;; platforms with 64-bit word size.
-#-sb-fluid (declaim (inline round-to-dualword))
-(defun round-to-dualword (size)
-  (logand (the word (+ size lowtag-mask)) (lognot lowtag-mask)))
+(defun instance-length (instance) ; excluding header, not aligned to even
+  ;; Add 1 if expressed length PLUS header (total number of words) would be
+  ;; an even number, and the hash state bits indicate hashed-and-moved.
+  (+ (%instance-length instance)
+     ;; Compute 1 or 0 depending whether the instance was physically extended
+     ;; by one word for the stable hash value. Extension occurs when and only when
+     ;; the hash state is hashed-and-moved, and the apparent total number of words
+     ;; inclusive of header (and exclusive of extension) is even. ANDing the least
+     ;; significant bit of the payload size with HASH-SLOT-PRESENT arrives at the
+     ;; desired boolean value. If apparent size is odd in hashed-and-moved state,
+     ;; the physical size undergoes no change.
+     (let ((header-word (instance-header-word instance)))
+       (logand (ash header-word (- instance-length-shift))
+               (ash header-word (- hash-slot-present-flag))
+               1))))
 
-;;; Return the vector OBJ, its WIDETAG, and the number of octets
-;;; required for its storage (including padding and alignment).
-(defun reconstitute-vector (obj saetp)
-  (declare (type (simple-array * (*)) obj)
-           (type specialized-array-element-type-properties saetp))
-  (let* ((length (+ (length obj)
-                    (saetp-n-pad-elements saetp)))
-         (n-bits (saetp-n-bits saetp))
-         (alignment-pad (floor 7 n-bits))
-         (n-data-octets (if (>= n-bits 8)
-                            (* length (ash n-bits -3))
-                            (ash (* (+ length alignment-pad)
-                                    n-bits)
-                                 -3))))
-    (values obj
-            (saetp-typecode saetp)
-            (round-to-dualword (+ (* vector-data-offset n-word-bytes)
-                                  n-data-octets)))))
+;;; * If symbols are 7 words incl header, as they are on 32-bit w/threads,
+;;;   then the part of NIL that manifests as symbol slots consumes 6 words
+;;;   (less the header) because NIL's symbol widetag precedes it by 1 word.
+;;;   So, there are 6 post-header words and 2 pre-header: one containing
+;;;   the widetag (not that it is ever read), and one 0 word.
+;;; * If symbols are 6 words incl header, as they are on 64-bit and
+;;;   32-bit w/o threads, then the symbol-like part of NIL is 5 words,
+;;;   which aligns up to 6, plus the two pre-header words.
+;;; So either way, NIL's alignment padding makes it come out to 8 words in total.
+(defconstant sizeof-nil-in-words (+ 2 (sb-int:align-up (1- sb-vm:symbol-size) 2)))
 
+;;; It's unclear to me whether reinventing sizetab[] in lisp is strictly
+;;; an improvement over doing the obvious:
+;;;  (WITH-PINNED-OBJECTS (object) (alien-funcall "ext_lispboj_size" ...))
+;;; It certainly doesn't feel that it's better, but it has significantly less overhead
+;;; at least on cheneygc if not also on the precise gencgc platforms.
+;;; But see WITH-PINNED-OBJECT-ITERATOR in 'target-hash-table' which offers
+;;; a way to mitigate the need to bind/unbind a special var when doing a massive
+;;; numbers of WITH-PINNED-OBJECT operations.
+;;; FIXME: export this from SB-EXT
 (defun primitive-object-size (object)
   "Return number of bytes of heap or stack directly consumed by OBJECT"
   (if (is-lisp-pointer (get-lisp-obj-address object))
       (let ((words
               (typecase object
                 (cons 2)
-                (instance (1+ (%instance-length object)))
+                (instance (1+ (instance-length object)))
                 (function
                  (when (= (fun-subtype object) simple-fun-widetag)
                    (return-from primitive-object-size
                      (primitive-object-size (fun-code-header object))))
                  (1+ (get-closure-length object)))
-                ;; NIL is larger than a symbol. I don't care to think about
-                ;; why these fudge factors are right, but they make the result
-                ;; equal to what MAP-ALLOCATED-OBJECTS reports.
-                (null (+ symbol-size 1 #+64-bit 1))
+                (null sizeof-nil-in-words)
                 (code-component
                  (return-from primitive-object-size (code-object-size object)))
                 (fdefn 4) ; no length stored in the header
@@ -223,76 +233,42 @@
                               (+ array-dimensions-offset (array-rank object)))
                              ((simple-array-nil-p object) 2)
                              (t
-                              (return-from primitive-object-size
-                                (nth-value 2 (reconstitute-vector object room-info)))))
+                              (let* ((length (+ (length object)
+                                                (saetp-n-pad-elements room-info)))
+                                     (n-bits (saetp-n-bits room-info))
+                                     (alignment-pad (floor 7 n-bits))
+                                     ;; This math confuses me. So there's a self-test
+                                     ;; against the C code which is known good.
+                                     (n-data-octets (if (>= n-bits 8)
+                                                        (* length (ash n-bits -3))
+                                                        (ash (* (+ length alignment-pad)
+                                                                n-bits)
+                                                             -3))))
+                                (+ (ceiling n-data-octets n-word-bytes) ; N data words
+                                   vector-data-offset))))
                        ;; Other things (symbol, value-cell, etc)
                        ;; don't have a sizer, so use GET-HEADER-DATA
                        (1+ (logand (get-header-data object)
-                                   (logand (get-header-data object)
-                                           (room-info-mask room-info))))))))))
+                                   (room-info-mask room-info)))))))))
         (* (align-up words 2) n-word-bytes))
       0))
 
-;;; Given the address (untagged, aligned, and interpreted as a FIXNUM)
-;;; of a lisp object, return the object, its "type code" (either
-;;; LIST-POINTER-LOWTAG or a header widetag), and the number of octets
-;;; required for its storage (including padding and alignment).  Note
-;;; that this function is designed to NOT CONS, even if called
-;;; out-of-line.
-;;; FIXME: size calculation should be via PRIMITIVE-OBJECT-SIZE, not reinvented
+(defmacro widetag@baseptr (sap)
+  #+big-endian `(sap-ref-8 ,sap ,(1- n-word-bytes))
+  #+little-endian `(sap-ref-8 ,sap 0))
+
+(defmacro lispobj@baseptr (sap widetag)
+  `(%make-lisp-obj
+    (logior (sap-int ,sap)
+            (logand (deref (extern-alien "widetag_lowtag" (array char 256)) ,widetag)
+                    lowtag-mask))))
+
+;;; This uses the funny fixnum representation of ADDRESS. I'd like to change this
+;;; to take a SAP but god forbid people are using it?
+;;; DO NOT USE THIS! It is soon to be removed
 (defun reconstitute-object (address)
-  (let* ((object-sap (int-sap (get-lisp-obj-address address)))
-         (header (sap-ref-word object-sap 0))
-         (widetag (logand header widetag-mask))
-         (header-value (ash header (- n-widetag-bits)))
-         (info (svref *room-info* widetag)))
-    (macrolet
-        ((boxed-size (header-value)
-           `(round-to-dualword (ash (1+ ,header-value) word-shift)))
-         (tagged-object (tag)
-           `(%make-lisp-obj (logior ,tag (get-lisp-obj-address address)))))
-      (cond
-          ;; Pick off arrays, as they're the only plausible cause for
-          ;; a non-nil, non-ROOM-INFO object as INFO.
-        ((specialized-array-element-type-properties-p info)
-         (reconstitute-vector (tagged-object other-pointer-lowtag) info))
-        ((= widetag filler-widetag)
-         (values nil filler-widetag (boxed-size header-value)))
-        ((null info)
-         (error "Unrecognized widetag #x~2,'0X in reconstitute-object"
-                widetag))
-
-        (t
-         (case (room-info-kind info)
-          (:list
-           (values (tagged-object list-pointer-lowtag)
-                   list-pointer-lowtag
-                   (* 2 n-word-bytes)))
-
-          (:instance
-           (values (tagged-object instance-pointer-lowtag)
-                   widetag
-                   (boxed-size (logand header-value short-header-max-words))))
-
-          (:closure ; also funcallable-instance
-           (values (tagged-object fun-pointer-lowtag)
-                   widetag
-                   (boxed-size (logand header-value short-header-max-words))))
-
-          (:code
-           (let ((c (tagged-object other-pointer-lowtag)))
-             (values c
-                     code-header-widetag
-                     (code-object-size c))))
-
-          (:fdefn
-           (values (tagged-object other-pointer-lowtag) widetag
-                   (* fdefn-size n-word-bytes)))
-
-          (:other
-           (values (tagged-object other-pointer-lowtag)
-                   widetag
-                   (boxed-size (logand header-value (room-info-mask info)))))))))))
+  (let ((sap (descriptor-sap address)))
+    (lispobj@baseptr sap (widetag@baseptr sap))))
 
 ;;; Iterate over all the objects in the contiguous block of memory
 ;;; with the low address at START and the high address just before
@@ -303,26 +279,47 @@
 (defun map-objects-in-range (fun start end &optional (strict-bound t))
   (declare (type function fun))
   (declare (dynamic-extent fun))
-  (named-let iter ((start start))
-  (cond
-    ((< (get-lisp-obj-address start) (get-lisp-obj-address end))
-     (multiple-value-bind (obj typecode size) (reconstitute-object start)
-      ;; SIZE is almost surely a fixnum. Non-fixnum would mean at least
-      ;; a 512MB object if 32-bit words, and is inconceivable if 64-bit.
-       (aver (not (logtest (the word size) lowtag-mask)))
+  (let ((start (descriptor-sap start))
+        (end (descriptor-sap end)))
+    (loop
+     (if (sap>= start end) (return))
+     (binding* ((widetag (widetag@baseptr start))
+                (obj (lispobj@baseptr start widetag))
+                ((typecode size)
+                 ;; PRIMITIVE-OBJECT-SIZE works on conses, but they're exceptions already
+                 ;; because of absence of a widetag, so may as well not call the sizer.
+                 (if (listp obj)
+                     (values list-pointer-lowtag (* 2 n-word-bytes))
+                     (values widetag (primitive-object-size obj)))))
+       ;; SIZE is surely a fixnum. Non-fixnum would imply at least
+       ;; a 512MB object if 32-bit words, and is inconceivable if 64-bit.
+       ;; But check to be sure.
+       (aver (not (logtest (the fixnum size) lowtag-mask)))
        (unless (= typecode filler-widetag)
          (funcall fun obj typecode size))
-             ;; This special little dance is to add a number of octets
-             ;; (and it had best be a number evenly divisible by our
-             ;; allocation granularity) to an unboxed, aligned address
-             ;; masquerading as a fixnum.  Without consing.
-      (iter (%make-lisp-obj
-              (mask-field (byte #.n-word-bits 0)
-                          (+ (get-lisp-obj-address start)
-                             size))))))
-    (strict-bound
+       (setq start (sap+ start size))))
+    (when strict-bound
      ;; If START is not eq to END, then we have blown past our endpoint.
-     (aver (eq start end))))))
+     (aver (sap= start end)))))
+
+;;; Test the sizing function ASAP, because if it's broken, then MAP-ALLOCATED-OBJECTS
+;;; is too, and then creating the initial core will crash because of the various heap
+;;; traversals performed in SAVE-LISP-AND-DIE. Don't delay a crash.
+(dovector (saetp *specialized-array-element-type-properties*)
+  (let ((length 0) (et (saetp-specifier saetp)))
+    (loop (let* ((array (make-array length :element-type et))
+                 (size-from-lisp (primitive-object-size array))
+                 (size-from-c
+                  (with-pinned-objects (array)
+                    (alien-funcall (extern-alien "ext_lispobj_size" (function unsigned unsigned))
+                                   (logandc2 (get-lisp-obj-address array) lowtag-mask)))))
+            (unless (= size-from-lisp size-from-c)
+              (bug "size calculation mismatch on ~S" array))
+            ;; Stop after enough trials to hit all the edge case
+            (when (or (>= size-from-lisp (* 8 sb-vm:n-word-bytes))
+                      (and (eq et nil) (>= length 4))) ; always 2 words
+              (return))
+            (incf length)))))
 
 ;;; Access to the GENCGC page table for better precision in
 ;;; MAP-ALLOCATED-OBJECTS
@@ -440,14 +437,9 @@ We could try a few things to mitigate this:
        ;; handling, as the header and alignment are slightly off.
        (multiple-value-bind (start end) (%space-bounds space)
          (declare (ignore start))
-         ;; This "8" is very magical. It happens to work for both
-         ;; word sizes, even though symbols differ in length
-         ;; (they can be either 6 or 7 words).
-         (funcall fun nil symbol-widetag (* 8 n-word-bytes))
-         ;; more magic: go to the next object following NIL, which works
-         ;; regardless of whether there is an object before NIL.
+         (funcall fun nil symbol-widetag (* sizeof-nil-in-words n-word-bytes))
          (let ((start (+ (logandc2 sb-vm:nil-value sb-vm:lowtag-mask)
-                         (ash 6 sb-vm:word-shift))))
+                         (ash (- sizeof-nil-in-words 2) sb-vm:word-shift))))
            (map-objects-in-range fun
                                  (ash start (- sb-vm:n-fixnum-tag-bits))
                                  end))))
@@ -498,7 +490,6 @@ We could try a few things to mitigate this:
        ;; prove to be an issue with concurrent systems, or with
        ;; spectacularly poor timing for closing an allocation region
        ;; in a single-threaded system.
-       #+sb-thread
        (close-current-gc-region)
        (do ((initial-next-free-page next-free-page)
             ;; This is a "funny" fixnum - essentially the bit cast of a pointer
@@ -572,7 +563,6 @@ We could try a few things to mitigate this:
         (without-gcing (do-1-space space))
         (do-1-space space)))))
 
-#+(and sb-thread gencgc)
 ;; Start with a Lisp rendition of ensure_region_closed() on the active
 ;; thread's region, since users are often surprised to learn that a
 ;; just-consed object can't necessarily be seen by MAP-ALLOCATED-OBJECTS.
@@ -582,15 +572,18 @@ We could try a few things to mitigate this:
 ;; by doing that.
 ;; (And seeing small consing by other threads is hopeless either way)
 (defun close-current-gc-region ()
-  (unless (eql (sap-int (current-thread-offset-sap
-                         (+ thread-alloc-region-slot 3)))
-               0)                       ; start_addr
-    (alien-funcall
-     (extern-alien "gc_close_region" (function void unsigned int))
-     (truly-the word
-                (+ (sb-thread::thread-primitive-thread sb-thread:*current-thread*)
-                   (ash thread-alloc-region-slot word-shift)))
-     1)))
+  #+gencgc
+  (let ((region-struct
+          #+sb-thread (sap+ (sb-thread::current-thread-sap)
+                            (ash thread-alloc-region-slot word-shift))
+          ;; If no threads, the alloc_region struct is in static space.
+          #-sb-thread (int-sap boxed-region)))
+    ;; The 'start_addr' field is at word index 3 of the structure (see gencgc-alloc-region.h).
+    (unless (eql (sap-ref-word region-struct (ash 3 word-shift)) 0)
+      (alien-funcall (extern-alien "gc_close_region" (function void system-area-pointer int))
+                     region-struct
+                     1))
+    nil))
 
 ;;;; MEMORY-USAGE
 
@@ -797,7 +790,7 @@ We could try a few things to mitigate this:
                  (eql type funcallable-instance-widetag))
          (incf total-objects)
          (let* ((layout (if (eql type funcallable-instance-widetag)
-                            (%funcallable-instance-layout obj)
+                            (%fun-layout obj)
                             (%instance-layout obj)))
                 (classoid (layout-classoid layout))
                 (found (ensure-gethash classoid totals (cons 0 0)))
@@ -1091,9 +1084,9 @@ We could try a few things to mitigate this:
                   ;; if functoid is a macro that does nothing.
                   (,functoid .o. ,@more)))
             ,.(make-case* 'funcallable-instance
-               `(let ((.l. (%funcallable-instance-layout ,obj)))
+               `(let ((.l. (%fun-layout ,obj)))
                   ;; As for INSTANCE, allow the functoid to see the access form
-                  (,functoid (%funcallable-instance-layout ,obj) ,@more)
+                  (,functoid (%fun-layout ,obj) ,@more)
                   (,functoid (%funcallable-instance-fun ,obj) ,@more)
                   (ecase (layout-bitmap .l.)
                     (#.sb-kernel:+layout-all-tagged+
@@ -1289,9 +1282,10 @@ We could try a few things to mitigate this:
            (let ((where (+ dynamic-space-start (* page-num gencgc-card-bytes)))
                  (seen-filler nil))
              (loop
-               (multiple-value-bind (obj type size)
-                   (reconstitute-object (ash where (- n-fixnum-tag-bits)))
-                 (when (= type code-header-widetag)
+               (let* ((obj (let ((sap (int-sap where)))
+                             (lispobj@baseptr sap (widetag@baseptr sap))))
+                      (size (primitive-object-size obj)))
+                 (when (code-component-p obj)
                    (incf n-code-bytes size))
                  (when (if (and (consp obj) (eq (car obj) 0) (eq (cdr obj) 0))
                            (if seen-filler
@@ -1429,7 +1423,12 @@ We could try a few things to mitigate this:
     (with-pinned-objects (obj)
       (dotimes (i count)
         (let ((word (sap-ref-word (int-sap addr) (ash i word-shift))))
-          (multiple-value-bind (lispobj ok) (if decode (make-lisp-obj word nil))
+          (multiple-value-bind (lispobj ok)
+              (cond ((and (typep thing 'code-component)
+                          (< 1 i (code-header-words thing)))
+                     (values (code-header-ref thing i) t))
+                    (decode
+                     (make-lisp-obj word nil)))
             (let ((*print-lines* 1)
                   (*print-pretty* t))
               (format t "~x: ~v,'0x~:[~; = ~S~]~%"
@@ -1457,6 +1456,11 @@ We could try a few things to mitigate this:
 (in-package "SB-C")
 ;;; As soon as practical in warm build it makes sense to add
 ;;; cold-allocation-point-fixups into the weak hash-table.
+;;; FIXME: I suspect that this wants to be just a weak vector
+;;; (all code objects that have any allocation profiling compiled in),
+;;; and not a hash-table, and that the list of fixups in the component
+;;; can be attached to the debug info (in the manner of debug funs).
+;;; When this was first implemented, weak-vectors weren't a thing. Maybe?
 (defvar *!cold-allocation-point-fixups*)
 (let ((hash-table (make-hash-table :test 'eq)))
   ;; Group by code component
