@@ -15,14 +15,14 @@
 
 ;;; Send the signal SIGNAL to the process with process id PID. SIGNAL
 ;;; should be a valid signal number
-#-sb-fluid (declaim (inline unix-kill))
+(declaim (inline unix-kill))
 (define-alien-routine ("kill" unix-kill) int
   (pid int)
   (signal int))
 
 ;;; Send the signal SIGNAL to the all the process in process group
 ;;; PGRP. SIGNAL should be a valid signal number
-#-sb-fluid (declaim (inline unix-killpg))
+(declaim (inline unix-killpg))
 (define-alien-routine ("killpg" unix-killpg) int
   (pgrp int)
   (signal int))
@@ -39,20 +39,11 @@
 
 #-sb-safepoint
 (defun unblock-gc-signals ()
-  (with-alien ((%unblock-gc-signals
-                (function void unsigned-long unsigned-long) :extern
-                "unblock_gc_signals"))
-    (alien-funcall %unblock-gc-signals 0 0)
+  (with-alien ((%unblock-gc-signals (function void) :extern "unblock_gc_signals"))
+    (alien-funcall %unblock-gc-signals)
     nil))
 
 
-;;;; C routines that actually do all the work of establishing signal handlers
-(define-alien-routine ("install_handler" install-handler)
-  unsigned-long
-  (signal int)
-  (handler unsigned-long)
-  (synchronous boolean))
-
 ;;;; interface to enabling and disabling signal handlers
 
 ;;; Note on the SYNCHRONOUS argument: On builds without pseudo-atomic,
@@ -68,36 +59,38 @@
 ;;; point instructions, certain system calls), hopefully ruling out the
 ;;; possibility that we would trigger it during allocation.
 
+;;; FIXME: terrible name. doesn't actually "enable" in the sense of unmasking.
 (defun enable-interrupt (signal handler &key synchronous)
-  (declare (type (or function fixnum (member :default :ignore)) handler))
-  (/show0 "enable-interrupt")
-  (flet ((run-handler (&rest args)
-           (declare (truly-dynamic-extent args))
+  (declare (type (or function (member :default :ignore)) handler))
+  (%install-handler signal handler synchronous)
+  ;; This used to return the previously installed handler, if any.
+  ;; It no longer does, but the old handler can be obtained via SAP-REF-LISPOBJ
+  ;; on 'lisp_sig_handlers'. The reason for not returning it is that the value was
+  ;; always a closure, and there's no easy/portable way to ask what function was
+  ;; really meant.  And you can't use that closure to reinstall the old thing
+  ;; because it would install a new closure on the old closure which, though
+  ;; feasible, is definitely not what was intended.
+  nil)
+
+(defun %install-handler (signal handler synchronous)
+  (flet ((run-handler (signo info-sap context-sap)
            #-(or c-stack-is-control-stack sb-safepoint) ;; able to do that in interrupt_handle_now()
            (unblock-gc-signals)
            (in-interruption ()
-             (apply handler args))))
-    (without-gcing
-      (let ((result (install-handler signal
-                                     (case handler
-                                       (:default sig-dfl)
-                                       (:ignore sig-ign)
-                                       (t
-                                        (sb-kernel:get-lisp-obj-address
-                                         #'run-handler)))
-                                     synchronous)))
-        (cond ((= result sig-dfl) :default)
-              ((= result sig-ign) :ignore)
-              (t ;; MAKE-LISP-OBJ returns 2 values, which gets
-                 ;; "too complex to check". We don't want the second value.
-               (values (the (or function fixnum)
-                         (sb-kernel:make-lisp-obj result)))))))))
-
-(defun default-interrupt (signal)
-  (enable-interrupt signal :default))
-
-(defun ignore-interrupt (signal)
-  (enable-interrupt signal :ignore))
+             (funcall handler signo info-sap context-sap))))
+    (with-pinned-objects (#'run-handler)
+      ;; 0 and 1 probably coincide with SIG_DFL and SIG_IGN, but those
+      ;; constants are opaque. We use our own explicit translation
+      ;; of them in the C install_handler() argument convention.
+      (with-alien ((%sigaction (function void int unsigned int) :extern "install_handler"))
+        (alien-funcall %sigaction
+                       signal
+                       (case handler
+                         (:default 0)
+                         (:ignore 1)
+                         (t (sb-kernel:get-lisp-obj-address #'run-handler)))
+                       (if synchronous 1 0)))))
+  nil)
 
 ;;;; Support for signal handlers which aren't.
 ;;;;
@@ -119,8 +112,7 @@
         (context (sap-ref-sap args sb-vm:n-word-bytes)))
     (dx-flet ((callback ()
                 (funcall run-handler signal info context)))
-      (sb-thread::initial-thread-function-trampoline thread nil
-                                                     #'callback nil))))
+      (sb-thread::run thread nil #'callback nil))))
 
 
 ;;;; default LISP signal handlers
@@ -144,7 +136,7 @@
                       (sap-int (sb-vm:context-pc context))))))))
 
 (define-signal-handler sigill-handler "illegal instruction")
-#-(or linux android)
+#-(or linux android haiku)
 (define-signal-handler sigemt-handler "SIGEMT")
 (define-signal-handler sigbus-handler "bus error")
 #-(or linux android)
@@ -214,25 +206,21 @@
 
 (defun sb-kernel:signal-cold-init-or-reinit ()
   "Enable all the default signals that Lisp knows how to deal with."
-  (enable-interrupt sigint #'sigint-handler)
-  (enable-interrupt sigterm #'sigterm-handler)
-  (enable-interrupt sigill #'sigill-handler :synchronous t)
-  #-(or linux android)
-  (enable-interrupt sigemt #'sigemt-handler)
-  (enable-interrupt sigfpe #'sb-vm:sigfpe-handler :synchronous t)
+  (%install-handler sigint #'sigint-handler nil)
+  (%install-handler sigterm #'sigterm-handler nil)
+  (%install-handler sigill #'sigill-handler t)
+   #-(or linux android haiku) (%install-handler sigemt #'sigemt-handler nil)
+  (%install-handler sigfpe #'sb-vm:sigfpe-handler t)
   (if (/= (extern-alien "install_sig_memory_fault_handler" int) 0)
-      (enable-interrupt sigbus #'sigbus-handler :synchronous t)
+      (%install-handler sigbus #'sigbus-handler t)
       (write-string ";;;; SIGBUS handler not installed
 " sb-sys:*stderr*))
-  #-(or linux android)
-  (enable-interrupt sigsys #'sigsys-handler :synchronous t)
-  #-sb-wtimer
-  (enable-interrupt sigalrm #'sigalrm-handler)
-  #-sb-thruption
-  (enable-interrupt sigpipe #'sigpipe-handler)
-  (enable-interrupt sigchld #'sigchld-handler)
-  #+hpux (ignore-interrupt sigxcpu)
+  #-(or linux android) (%install-handler sigsys #'sigsys-handler t)
+  #-sb-wtimer (%install-handler sigalrm #'sigalrm-handler nil)
+  #-sb-thruption (%install-handler sigpipe #'sigpipe-handler nil)
+  (%install-handler sigchld #'sigchld-handler nil)
   #-sb-safepoint (unblock-gc-signals)
+  ;; I don't see why deferrables would have ben blocked. This is essentially a no-op.
   (unblock-deferrable-signals)
   (values))
 
