@@ -197,10 +197,10 @@
            (unuse-package ext "CL-USER")))
 
 #+cmu
-(setq *compile-print* nil) ; too much noise, can't see the actual warnings
+(setq cl:*compile-print* nil) ; too much noise, can't see the actual warnings
 #+sbcl
 (progn
-  (setq *compile-print* nil)
+  (setq cl:*compile-print* nil)
   (load "src/cold/muffler.lisp")
   ;; Let's just say we never care to see these.
   (declaim (sb-ext:muffle-conditions
@@ -243,6 +243,12 @@
              (gc (find-if (lambda (x) (member x '(:cheneygc :gencgc)))
                           target-feature-list))
              (arch (target-platform-keyword target-feature-list)))
+        (when (featurep '(:and :sb-thread (:or :linux :win32)) target-feature-list)
+          (pushnew :sb-futex target-feature-list))
+        (when (featurep '(:and :sb-thread (:not :win32)) target-feature-list)
+          (push :pauseless-threadstart target-feature-list))
+        (when (featurep '(:and :sb-thread (:or :darwin :openbsd)) target-feature-list)
+          (push :os-thread-stack target-feature-list))
         (when (and (member :x86 target-feature-list)
                    (member :int4-breakpoints target-feature-list))
           ;; 0xCE is a perfectly good 32-bit instruction,
@@ -287,6 +293,8 @@
 (let ((feature-compatibility-tests
        '(("(and sb-thread (not gencgc))"
           ":SB-THREAD requires :GENCGC")
+         ("(and pauseless-threadstart (not sb-thread))"
+          ":PAUSELESS-THREADSTART requires :SB-THREAD")
          ("(and sb-thread (not (or riscv ppc ppc64 x86 x86-64 arm64)))"
           ":SB-THREAD not supported on selected architecture")
          ("(and gencgc cheneygc)"
@@ -320,6 +328,8 @@
           ":IMMOBILE-CODE requires :IMMOBILE-SPACE feature")
          ("(and immobile-symbols (not immobile-space))"
           ":IMMOBILE-SYMBOLS requires :IMMOBILE-SPACE feature")
+         ("(and sb-futex (not sb-thread))"
+          "Can't enable SB-FUTEX on platforms lacking thread support")
          ;; There is still hope to make multithreading on DragonFly x86-64
          ("(and sb-thread x86 dragonfly)"
           ":SB-THREAD not supported on selected architecture")))
@@ -402,18 +412,12 @@
     ;; just to avoid that ugly mess.
     nil))
 
-;;; The specialized array registry has file-wide scope. Hacking that aspect
-;;; into the xc build scaffold seemed slightly easier than hacking the
-;;; compiler (i.e. making the registry a slot of the fasl-output struct)
-(defvar *array-to-specialization* (make-hash-table :test #'eq))
-
 (defmacro do-stems-and-flags ((stem flags build-phase) &body body)
   (let ((stem-and-flags (gensym "STEM-AND-FLAGS")))
     `(dolist (,stem-and-flags (get-stems-and-flags ,build-phase))
        (let ((,stem (first ,stem-and-flags))
              (,flags (rest ,stem-and-flags)))
-         ,@body
-         (clrhash *array-to-specialization*)))))
+         ,@body))))
 
 ;;; Given a STEM, remap the path component "/target/" to a suitable
 ;;; target directory.
@@ -544,7 +548,7 @@
          (ignore-failure-p (find :ignore-failure-p flags)))
     (declare (type function compile-file))
 
-    (ensure-directories-exist obj :verbose *compile-print*) ; host's value
+    (ensure-directories-exist obj :verbose cl:*compile-print*) ; host's value
 
     ;; We're about to set about building a new object file. First, we
     ;; delete any preexisting object file in order to avoid confusing
@@ -592,10 +596,7 @@
        retry-compile-file
          (multiple-value-bind (output-truename warnings-p failure-p)
              (restart-case
-                 (funcall compile-file src :output-file tmp-obj
-                          ;; if tracing, also show high-level progress
-                          :trace-file trace-file
-                          :print trace-file
+                 (apply compile-file src :output-file tmp-obj
                           :block-compile (and ;; Block compilation was
                                               ;; completely broken
                                               ;; from the beginning of
@@ -609,7 +610,12 @@
                                                                          (lisp-implementation-version))
                                                                 '(2 0 2))))
                                               block-compile)
-                          :allow-other-keys t)
+                          :allow-other-keys t
+                          ;; If tracing, also print, but don't specify :PRINT unless specifying
+                          ;; :TRACE-FILE so that whatever the default is for *COMPILE-PRINT*
+                          ;; prevails, insensitively to whether it's the SB-XC: or CL: symbol.
+                          (when trace-file
+                            '(:trace-file t :print t)))
                (recompile ()
                  :report report-recompile-restart
                  (go retry-compile-file)))
