@@ -33,8 +33,10 @@
   ;; whatever it is that show would have shown. Comment this out if you need.
     (when s (set s nil))))
 
-(assert (zerop (deref (extern-alien "lowtag_for_widetag" (array char 64))
-                      (ash sb-vm:character-widetag -2))))
+(let ((byte (deref (extern-alien "widetag_lowtag" (array char 256))
+                   sb-vm:character-widetag)))
+  (assert (not (logbitp 7 byte))) ; not a headered object
+  (assert (= (logand byte sb-vm:lowtag-mask) sb-vm:list-pointer-lowtag)))
 (gc :full t)
 
 ;;; Verify that all defstructs except for one were compiled in a null lexical
@@ -143,9 +145,20 @@
               retry-compile-file
                 (multiple-value-bind (output-truename warnings-p failure-p)
                     (ecase (if (boundp '*compile-files-p*) *compile-files-p* t)
-                     ((t)   (let ((sb-c::*source-namestring* fullname))
-                              (ensure-directories-exist output)
-                              (compile-file stem :output-file output)))
+                     ((t)
+                      (let ((sb-c::*source-namestring* fullname)
+                            (sb-ext:*derive-function-types*
+                              (unless (search "/pcl/" stem)
+                                t)))
+                        (ensure-directories-exist output)
+                        ;; Like PROCLAIM-TARGET-OPTIMIZATION in 'compile-cold-sbcl'
+                        ;; We should probably stash a copy of the POLICY instance from
+                        ;; make-host-2 in a global var and apply it here.
+                        (proclaim '(optimize
+                                    (safety 2) (speed 2)
+                                    (sb-c:insert-step-conditions 0)
+                                    (sb-c:alien-funcall-saves-fp-and-pc #+x86 3 #-x86 0)))
+                        (compile-file stem :output-file output)))
                      ((nil) output))
                   (cond ((not output-truename)
                          (error "COMPILE-FILE of ~S failed." stem))
@@ -178,7 +191,7 @@
                     (error "LOAD of ~S failed." output-truename))
                   (sb-int:/show "done loading" output-truename))))))))
 
-  (let ((*compile-print* nil))
+  (let ((cl:*compile-print* nil))
     (dolist (group sources)
       (handler-bind ((#+x86-64 warning #-x86-64 simple-warning
                       (lambda (c)
@@ -188,7 +201,16 @@
                                    (search "undefined variable"
                                            (write-to-string c :escape nil)))
                           (cerror "Finish warm compile ignoring the problem" c)))))
-        (with-compilation-unit () (do-srcs group)))))))
+        (with-compilation-unit ()
+          (do-srcs group)
+          ;; I do not know why #+sb-show gets several "undefined-type CLASS" warnings
+          ;; that #-sb-show doesn't. And CLASS is a reserved name not defined as yet,
+          ;; so we have to pretend that didn't happen, otherwise the warning about
+          ;; not being able to define CLASS as a type name breaks the build.
+          #+sb-show
+          (setq sb-c::*undefined-warnings*
+                (delete 'class sb-c::*undefined-warnings*
+                        :key #'sb-c::undefined-warning-name))))))))
 
 (when (hash-table-p sb-c::*static-vop-usage-counts*)
   (with-open-file (output "output/warm-vop-usage.txt"
