@@ -69,7 +69,7 @@
 
 ;;; The LEXENV represents the lexical environment used for IR1 conversion.
 ;;; (This is also what shows up as an ENVIRONMENT value in macroexpansion.)
-#-sb-fluid (declaim (inline internal-make-lexenv)) ; only called in one place
+(declaim (inline internal-make-lexenv)) ; only called in one place
 (defstruct (lexenv
             (:include abstract-lexenv)
              (:print-function
@@ -139,7 +139,7 @@
   ;; are of the form (:declare name . value),
   ;; (:variable name key . value), or (:function name key . value)
   (user-data nil :type list)
-  parent
+  (parent nil)
   ;; Cache of all visible variables, including the ones coming from
   ;; (call-lexenv lambda)
   ;; Used for LEAF-VISIBLE-TO-DEBUGGER-P
