@@ -11,12 +11,17 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ; ok, so we're supposed to use our instruction scheduler, but we don't,
 ; because of "needs a little more work in the assembler"
-(defconstant sb!assem:assem-scheduler-p nil)
-(defconstant sb!assem:+inst-alignment-bytes+ 4)
+(defconstant sb-assem:assem-scheduler-p nil)
+(defconstant sb-assem:+inst-alignment-bytes+ 4)
+;;; needs a little more work in the assembler, to realise that the
+;;; delays requested here are not mandatory, so that the assembler
+;;; shouldn't fill gaps with NOPs but with real instructions.  -- CSR,
+;;; 2003-09-08
+#+nil (defconstant sb-assem:+assem-max-locations+ 70)
 
 (defconstant +backend-fasl-file-implementation+ :ppc)
   ;; On Linux, the ABI specifies the page size to be 4k-64k, use the
@@ -24,7 +29,7 @@
   ;; find out whether using exact multiples of the page size actually
   ;; matters in the few places where that's done, or whether we could
   ;; just use 4k everywhere.
-(defconstant +backend-page-bytes+ #!+linux 65536 #!-linux 4096)
+(defconstant +backend-page-bytes+ #+linux 65536 #-linux 4096)
 
 ;;; The size in bytes of GENCGC cards, i.e. the granularity at which
 ;;; writes to old generations are logged.  With mprotect-based write
@@ -58,7 +63,6 @@
 (defconstant single-float-normal-exponent-min 1)
 (defconstant single-float-normal-exponent-max 254)
 (defconstant single-float-hidden-bit (ash 1 23))
-(defconstant single-float-trapping-nan-bit (ash 1 22))
 
 (defconstant double-float-bias 1022)
 (defconstant-eqx double-float-exponent-byte (byte 11 20) #'equalp)
@@ -66,7 +70,6 @@
 (defconstant double-float-normal-exponent-min 1)
 (defconstant double-float-normal-exponent-max #x7FE)
 (defconstant double-float-hidden-bit (ash 1 20))
-(defconstant double-float-trapping-nan-bit (ash 1 19))
 
 (defconstant single-float-digits
   (+ (byte-size single-float-significand-byte) 1))
@@ -110,7 +113,7 @@
 ;;;; Where to put the different spaces.
 
 ;;; On non-gencgc we need large dynamic and static spaces for PURIFY
-#!-gencgc
+#-gencgc
 (progn
   (defconstant read-only-space-start #x04000000)
   (defconstant read-only-space-end   #x07ff8000)
@@ -121,26 +124,28 @@
   (defconstant linkage-table-space-end   #x0b000000))
 
 ;;; While on gencgc we don't.
-#!+gencgc
+#+gencgc
 (!gencgc-space-setup #x04000000
+                     :read-only-space-size 0
                      :dynamic-space-start
-                     #!+linux   #x4f000000
-                     #!+netbsd  #x4f000000
-                     #!+openbsd #x4f000000
-                     #!+darwin  #x10000000)
+                     #+linux   #x4f000000
+                     #+netbsd  #x4f000000
+                     #+openbsd #x4f000000
+                     #+darwin  #x10000000)
 
+(defconstant linkage-table-growth-direction :up)
 (defconstant linkage-table-entry-size 16)
 
-#!+linux
+#+linux
 (progn
-  #!-gencgc
+  #-gencgc
   (progn
     (defparameter dynamic-0-space-start #x4f000000)
     (defparameter dynamic-0-space-end   #x66fff000)))
 
-#!+netbsd
+#+netbsd
 (progn
-  #!-gencgc
+  #-gencgc
   (progn
     (defparameter dynamic-0-space-start #x4f000000)
     (defparameter dynamic-0-space-end   #x66fff000)))
@@ -152,16 +157,16 @@
 ;;; FIXME: MAXDSIZ is a kernel parameter, and can vary as high as 1GB.
 ;;; These parameters should probably be tested under such a configuration,
 ;;; as rare as it might or might not be.
-#!+openbsd
+#+openbsd
 (progn
-  #!-gencgc
+  #-gencgc
   (progn
     (defparameter dynamic-0-space-start #x4f000000)
     (defparameter dynamic-0-space-end   #x5cfff000)))
 
-#!+darwin
+#+darwin
 (progn
-  #!-gencgc
+  #-gencgc
   (progn
     (defparameter dynamic-0-space-start #x10000000)
     (defparameter dynamic-0-space-end   #x3ffff000)))
