@@ -34,9 +34,82 @@
   (:info name dx-p offset lowtag)
   (:ignore name dx-p))
 
+(define-vop (compare-and-swap-slot)
+  (:args (object :scs (descriptor-reg))
+         (old :scs (descriptor-reg any-reg))
+         (new :scs (descriptor-reg any-reg)))
+  (:temporary (:sc non-descriptor-reg) temp)
+  (:info name offset lowtag)
+  (:ignore name)
+  (:temporary (:sc interior-reg) lip)
+  (:results (result :scs (descriptor-reg) :from :load))
+  (:generator 5
+    (inst addi lip object (- (* offset n-word-bytes) lowtag))
+    LOOP
+    (inst lr result lip :aq)
+    (inst bne result old EXIT)
+    (inst sc temp new lip :aq :rl)
+    (inst bne temp zero-tn LOOP)
+    EXIT))
+
 ;;;; Symbol hacking VOPs:
+(define-vop (%compare-and-swap-symbol-value)
+  (:translate %compare-and-swap-symbol-value)
+  (:args (symbol :scs (descriptor-reg))
+         (old :scs (descriptor-reg any-reg))
+         (new :scs (descriptor-reg any-reg)))
+  (:temporary (:sc non-descriptor-reg) temp)
+  (:temporary (:sc interior-reg) lip)
+  (:results (result :scs (descriptor-reg any-reg)
+                    :from :load))
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 15
+    #+sb-thread
+    (assemble ()
+      (load-tls-index temp symbol)
+      (inst add lip thread-base-tn temp)
+      ;; Thread-local area, no synchronization needed.
+      (loadw result lip)
+      (inst bne result old DONT-STORE-TLS)
+      (storew new lip)
+      DONT-STORE-TLS
+      (inst xori temp result no-tls-value-marker-widetag)
+      (inst bne temp zero-tn CHECK-UNBOUND))
+
+    (inst addi lip symbol (- (* symbol-value-slot n-word-bytes)
+                             other-pointer-lowtag))
+    LOOP
+    (inst lr result lip :aq)
+    (inst bne result old CHECK-UNBOUND)
+    (inst sc temp new lip :aq :rl)
+    (inst bne temp zero-tn LOOP)
+
+    CHECK-UNBOUND
+    (inst xori temp result unbound-marker-widetag)
+    (inst beq temp zero-tn (generate-error-code vop 'unbound-symbol-error symbol))))
 
 ;;; The compiler likes to be able to directly SET symbols.
+#+sb-thread
+(define-vop (set)
+  (:args (symbol :scs (descriptor-reg))
+         (value :scs (descriptor-reg any-reg)))
+  (:temporary (:scs (non-descriptor-reg)) temp)
+  (:temporary (:sc any-reg) tls-slot)
+  (:temporary (:sc interior-reg) lip)
+  (:generator 4
+    (load-tls-index tls-slot symbol)
+    (inst add lip thread-base-tn tls-slot)
+    (loadw temp lip)
+    (inst xori temp temp no-tls-value-marker-widetag)
+    (inst bne temp zero-tn TLS-VALUE)
+    (storew value symbol symbol-value-slot other-pointer-lowtag)
+    (inst j DONE)
+    TLS-VALUE
+    (storew value lip)
+    DONE))
+
+#-sb-thread
 (define-vop (set cell-set)
   (:variant symbol-value-slot other-pointer-lowtag))
 
@@ -51,6 +124,26 @@
 
 ;;; With Symbol-Value, we check that the value isn't the trap object.
 ;;; So Symbol-Value of NIL is NIL.
+#+sb-thread
+(define-vop (symbol-value checked-cell-ref)
+  (:translate symeval)
+  (:temporary (:scs (interior-reg)) lip)
+  (:variant-vars check-boundp)
+  (:variant t)
+  (:generator 9
+    (load-tls-index value object)
+    (inst add lip thread-base-tn value)
+    (loadw value lip)
+    (inst xori temp value no-tls-value-marker-widetag)
+    (inst bne temp zero-tn CHECK-UNBOUND)
+    (loadw value object symbol-value-slot other-pointer-lowtag)
+    CHECK-UNBOUND
+    (when check-boundp
+      (let ((err-lab (generate-error-code vop 'unbound-symbol-error object)))
+        (inst xori temp value unbound-marker-widetag)
+        (inst beq temp zero-tn err-lab)))))
+
+#-sb-thread
 (define-vop (symbol-value checked-cell-ref)
   (:translate symeval)
   (:generator 9
@@ -68,6 +161,24 @@
   (:temporary (:scs (descriptor-reg)) value)
   (:temporary (:scs (non-descriptor-reg)) temp))
 
+#+sb-thread
+(define-vop (boundp boundp-frob)
+  (:temporary (:scs (interior-reg)) lip)
+  (:translate boundp)
+  (:generator 9
+    (load-tls-index value object)
+    (inst add lip thread-base-tn value)
+    (loadw value lip)
+    (inst xori temp value no-tls-value-marker-widetag)
+    (inst bne temp zero-tn CHECK-UNBOUND)
+    (loadw value object symbol-value-slot other-pointer-lowtag)
+    CHECK-UNBOUND
+    (inst xori temp value unbound-marker-widetag)
+    (if not-p
+        (inst beq temp zero-tn target)
+        (inst bne temp zero-tn target))))
+
+#-sb-thread
 (define-vop (boundp boundp-frob)
   (:translate boundp)
   (:generator 9
@@ -77,6 +188,13 @@
         (inst beq temp zero-tn target)
         (inst bne temp zero-tn target))))
 
+#+sb-thread
+(define-vop (fast-symbol-value symbol-value)
+  (:policy :fast)
+  (:variant nil)
+  (:variant-cost 7))
+
+#-sb-thread
 (define-vop (fast-symbol-value cell-ref)
   (:variant symbol-value-slot other-pointer-lowtag)
   (:policy :fast)
@@ -157,6 +275,33 @@
 ;;; BIND -- Establish VAL as a binding for SYMBOL.  Save the old value and
 ;;; the symbol on the binding stack and stuff the new value into the
 ;;; symbol.
+#+sb-thread
+(define-vop (dynbind)
+  (:args (value :scs (any-reg descriptor-reg))
+         (symbol :scs (descriptor-reg) :target alloc-tls-symbol))
+  ;; These have a dual personality in the assembly routine. We are
+  ;; trying to pack as tightly as possible.
+  (:temporary (:scs (descriptor-reg) :offset l0-offset) alloc-tls-symbol)
+  (:temporary (:scs (descriptor-reg) :offset l1-offset) value-temp)
+  (:temporary (:scs (non-descriptor-reg) :offset nl0-offset) tls-index)
+  (:temporary (:scs (non-descriptor-reg) :offset nl1-offset) bsp-temp)
+  (:temporary (:scs (interior-reg)) lip)
+  (:generator 5
+     (load-tls-index tls-index symbol)
+     (inst bne tls-index zero-tn TLS-VALID)
+     (move alloc-tls-symbol symbol)
+     (invoke-asm-routine 'alloc-tls-index)
+     TLS-VALID
+     (inst add lip thread-base-tn tls-index)
+     (loadw value-temp lip)
+     (storew value lip)
+     (load-binding-stack-pointer bsp-temp)
+     (inst addi bsp-temp bsp-temp (* binding-size n-word-bytes))
+     (store-binding-stack-pointer bsp-temp)
+     (storew value-temp bsp-temp (- binding-value-slot binding-size))
+     (storew tls-index bsp-temp (- binding-symbol-slot binding-size))))
+
+#-sb-thread
 (define-vop (dynbind)
   (:args (value :scs (any-reg descriptor-reg))
          (symbol :scs (descriptor-reg)))
@@ -171,6 +316,23 @@
     (storew symbol bsp-temp (- binding-symbol-slot binding-size))
     (storew value symbol symbol-value-slot other-pointer-lowtag)))
 
+#+sb-thread
+(define-vop (unbind)
+  (:temporary (:scs (descriptor-reg)) value)
+  (:temporary (:scs (any-reg)) tls-index bsp-temp)
+  (:temporary (:scs (interior-reg)) lip)
+  (:generator 0
+    (load-binding-stack-pointer bsp-temp)
+    (loadw tls-index bsp-temp (- binding-symbol-slot binding-size))
+    (loadw value bsp-temp (- binding-value-slot binding-size))
+    (inst add lip thread-base-tn tls-index)
+    (storew value lip)
+    (storew zero-tn bsp-temp (- binding-symbol-slot binding-size))
+    (storew zero-tn bsp-temp (- binding-value-slot binding-size))
+    (inst subi bsp-temp bsp-temp (* binding-size n-word-bytes))
+    (store-binding-stack-pointer bsp-temp)))
+
+#-sb-thread
 (define-vop (unbind)
   (:temporary (:scs (descriptor-reg)) symbol value)
   (:temporary (:scs (any-reg)) bsp-temp)
@@ -189,6 +351,8 @@
   (:temporary (:scs (any-reg) :from (:argument 0)) where)
   (:temporary (:scs (descriptor-reg)) symbol value)
   (:temporary (:scs (any-reg)) bsp)
+  #+sb-thread
+  (:temporary (:scs (interior-reg)) lip)
   (:generator 0
     (load-binding-stack-pointer bsp)
     (move where arg)
@@ -198,7 +362,12 @@
     (loadw symbol bsp (- binding-symbol-slot binding-size))
     (inst beq symbol zero-tn skip)
     (loadw value bsp (- binding-value-slot binding-size))
+    #-sb-thread
     (storew value symbol symbol-value-slot other-pointer-lowtag)
+    #+sb-thread
+    (progn
+      (inst add lip thread-base-tn symbol)
+      (storew value lip))
     (storew zero-tn bsp (- binding-symbol-slot binding-size))
 
     SKIP
@@ -260,7 +429,7 @@
   (:result-types positive-fixnum)
   (:generator 4
     (loadw res struct 0 instance-pointer-lowtag)
-    (inst srli res res n-widetag-bits)))
+    (inst srli res res instance-length-shift)))
 
 (define-full-reffer instance-index-ref * instance-slots-offset
   instance-pointer-lowtag (descriptor-reg any-reg) * %instance-ref)
@@ -268,6 +437,10 @@
 (define-full-setter instance-index-set * instance-slots-offset
   instance-pointer-lowtag (descriptor-reg any-reg) * %instance-set)
 
+(define-full-casser instance-index-cas * instance-slots-offset
+  instance-pointer-lowtag (descriptor-reg any-reg) * %instance-cas)
+
+
 ;;;; Code object frobbing.
 
 (define-full-reffer code-header-ref * 0 other-pointer-lowtag
@@ -285,7 +458,10 @@
                   ,(symbolicate "%RAW-INSTANCE-REF/" name))
                 (define-full-setter ,(symbolicate "RAW-INSTANCE-SET/" name) * instance-slots-offset
                   instance-pointer-lowtag (,value-sc) ,value-primtype
-                  ,(symbolicate "%RAW-INSTANCE-SET/" name)))))
+                  ,(symbolicate "%RAW-INSTANCE-SET/" name))
+                (define-full-casser ,(symbolicate "RAW-INSTANCE-CAS/" name) instance instance-slots-offset
+                  instance-pointer-lowtag (,value-sc) ,value-primtype
+                  ,(symbolicate "%RAW-INSTANCE-CAS/" name)))))
   (define-raw-slot-word-vops word unsigned-reg unsigned-num)
   (define-raw-slot-word-vops signed-word signed-reg signed-num))
 
@@ -309,3 +485,6 @@
   (define-raw-slot-float-vops double double-float double-reg 8 :double)
   (define-raw-slot-float-vops complex-single complex-single-float complex-single-reg 4 :single t)
   (define-raw-slot-float-vops complex-double complex-double-float complex-double-reg 8 :double t))
+
+(define-atomic-frobber raw-instance-incf/word amoadd * instance-slots-offset
+  instance-pointer-lowtag (unsigned-reg) unsigned-num  %raw-instance-atomic-incf/word)
