@@ -48,7 +48,7 @@
          (p2 (%simd-pack-256-2 x))
          (p3 (%simd-pack-256-3 x)))
     (cond ((= p0 p1 p2 p3 0)
-           (inst vxorps y y))
+           (inst vxorps y y y))
           ((= p0 p1 p2 p3 (ldb (byte 64 0) -1))
            (inst vpcmpeqd y y y))
           (t
@@ -98,7 +98,7 @@
                 (int-avx2-reg 0)
                 (t 0)))
          y simd-pack-256-tag-slot other-pointer-lowtag)
-     (let ((ea (make-ea-for-object-slot
+     (let ((ea (object-slot-ea
                 y simd-pack-256-p0-slot other-pointer-lowtag)))
        (if (float-avx2-p x)
            (inst vmovups ea x)
@@ -111,7 +111,7 @@
   (:results (y :scs (int-avx2-reg double-avx2-reg single-avx2-reg)))
   (:note "pointer to AVX2 coercion")
   (:generator 2
-    (let ((ea (make-ea-for-object-slot x simd-pack-256-p0-slot other-pointer-lowtag)))
+    (let ((ea (object-slot-ea x simd-pack-256-p0-slot other-pointer-lowtag)))
       (if (float-avx2-p y)
           (inst vmovups y ea)
           (inst vmovdqu y ea)))))
@@ -370,4 +370,10 @@
   (values (%simd-pack-256-double-item pack 0)
           (%simd-pack-256-double-item pack 1)
           (%simd-pack-256-double-item pack 2)
-          (%simd-pack-256-double-item pack 3))))
+          (%simd-pack-256-double-item pack 3)))
+
+(defun %simd-pack-256-inline-constant (pack)
+  (list :avx2 (logior (%simd-pack-256-0 pack)
+                      (ash (%simd-pack-256-1 pack) 64)
+                      (ash (%simd-pack-256-2 pack) 128)
+                      (ash (%simd-pack-256-3 pack) 192)))))
