@@ -280,10 +280,9 @@ os_install_interrupt_handlers(void)
     SHOW("os_install_interrupt_handlers()/bsd-os/defined(GENCGC)");
     if (INSTALL_SIG_MEMORY_FAULT_HANDLER) {
 #if defined(LISP_FEATURE_MACH_EXCEPTION_HANDLER)
-    undoably_install_low_level_interrupt_handler(SIG_MEMORY_FAULT,
-                                                 mach_error_memory_fault_handler);
+    ll_install_handler(SIG_MEMORY_FAULT, mach_error_memory_fault_handler);
 #else
-    undoably_install_low_level_interrupt_handler(SIG_MEMORY_FAULT,
+    ll_install_handler(SIG_MEMORY_FAULT,
 #if defined(LISP_FEATURE_FREEBSD) && !defined(__GLIBC__)
                                                  (__siginfohandler_t *)
 #endif
@@ -294,13 +293,13 @@ os_install_interrupt_handlers(void)
 #ifdef LISP_FEATURE_SB_THREAD
 # ifdef LISP_FEATURE_SB_SAFEPOINT
 #  ifdef LISP_FEATURE_SB_THRUPTION
-    undoably_install_low_level_interrupt_handler(SIGPIPE, thruption_handler);
+    ll_install_handler(SIGPIPE, thruption_handler);
 #  endif
 # else
-    undoably_install_low_level_interrupt_handler(SIG_STOP_FOR_GC,
-                                                 sig_stop_for_gc_handler);
+    ll_install_handler(SIG_STOP_FOR_GC, sig_stop_for_gc_handler);
 # endif
 #endif
+
     SHOW("leaving os_install_interrupt_handlers()");
 }
 
@@ -321,8 +320,7 @@ void
 os_install_interrupt_handlers(void)
 {
     SHOW("os_install_interrupt_handlers()/bsd-os/!defined(GENCGC)");
-    undoably_install_low_level_interrupt_handler(SIG_MEMORY_FAULT,
-                                                 sigsegv_handler);
+    ll_install_handler(SIG_MEMORY_FAULT, sigsegv_handler);
 }
 
 #endif /* defined GENCGC */
