@@ -17,13 +17,64 @@
  */
 
 #include "os.h"
-#include "gc-internal.h" // for os_allocate()
+#include "gc-internal.h" // for sizetab[] and os_allocate()
 #include "hopscotch.h"
 #include <stdint.h>
 #include <stdio.h>
+#ifdef LISP_FEATURE_WIN32
+/* I don't know where ffs() is prototyped */
+extern int ffs(int);
+#else
+/* https://www.freebsd.org/cgi/man.cgi?query=fls&sektion=3&manpath=FreeBSD+7.1-RELEASE
+   says strings.h */
+#include <strings.h>
+#endif
 #include "genesis/vector.h"
 #include "murmur_hash.h"
 
+#ifdef LISP_FEATURE_USE_SYS_MMAP
+///
+///                **********************************
+///                *  GC MUST NOT ACQUIRE ANY LOCKS *
+///                **********************************
+///
+/// It's normally fine to use the mmap() system call to obtain memory for the
+/// hopscotch tables, unless your C runtime intercepts mmap() and causes it
+/// to sometimes (or always?) need a spinlock. That lock may be owned already,
+/// so GC will patiently wait forever; meanwhile the lock owner is also
+/// waiting forever on GC to finish.
+/// So bypass the C library routine and call the OS directly
+/// in case of a non-signal-safe interceptor such as
+///   https://chromium.googlesource.com/chromium/src/third_party/tcmalloc/chromium/+/refs/heads/master/src/malloc_hook_mmap_linux.h#146
+///
+static inline void* sys_mmap(void* addr, size_t length, int prot, int flags,
+                             int fd, off_t offset) {
+    // "linux-os.h" brings in <syscall.h>, others may need something different.
+    // mmap2 allows large file access with 32-bit off_t. We don't care about that,
+    // but _usually_ only one or the other of the syscalls exists depending on,
+    // various factors. Basing it on word size will pick the right one.
+#ifdef LISP_FEATURE_64_BIT
+    void* result = (void*)syscall(SYS_mmap, addr, length, prot, flags, fd, offset);
+#else
+    void* result = (void*)syscall(SYS_mmap2, addr, length, prot, flags, fd, offset);
+#endif
+    if (result == MAP_FAILED) lose("mmap failed in GC");
+    return result;
+}
+static inline int sys_munmap(void* addr, size_t length) {
+    return syscall(__NR_munmap, addr, length);
+}
+#define hopscotch_allocate(nbytes) \
+  sys_mmap(0, nbytes, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, 0, 0)
+#define hopscotch_deallocate(addr,length) sys_munmap(addr, length)
+
+#else
+
+#define hopscotch_allocate(nbytes) os_allocate(nbytes)
+#define hopscotch_deallocate(addr,length) os_deallocate(addr, length)
+
+#endif
+
 typedef struct hopscotch_table* tableptr;
 void hopscotch_integrity_check(tableptr,char*,int);
 
@@ -123,7 +174,7 @@ void hopscotch_init() // Called once on runtime startup, from gc_init().
     // Prefill the cache with 2 entries, each the size of a kernel page.
     int n_bytes_per_slice = getpagesize();
     int n_bytes_total = N_CACHED_ALLOCS * n_bytes_per_slice;
-    char* mem = os_allocate(n_bytes_total);
+    char* mem = hopscotch_allocate(n_bytes_total);
     gc_assert(mem);
     cached_alloc[0] = mem + ALLOCATION_OVERHEAD;
     cached_alloc[1] = cached_alloc[0] + n_bytes_per_slice;
@@ -158,7 +209,7 @@ static char* cached_allocate(os_vm_size_t nbytes)
     // not a multiple of the mmap granularity, which we'll assume is 4K.
     // (It doesn't actually matter.)
     nbytes = ALIGN_UP(nbytes, hh_allocation_granularity);
-    char* result = os_allocate(nbytes);
+    char* result = hopscotch_allocate(nbytes);
     gc_assert(result);
     result += ALLOCATION_OVERHEAD;
     usable_size(result) = nbytes - ALLOCATION_OVERHEAD;
@@ -171,7 +222,7 @@ static char* cached_allocate(os_vm_size_t nbytes)
  * which is why the length is specified again.
  * If returning it to the OS and not the cache, then don't bother 0-filling.
  */
-static void cached_deallocate(char* mem, int zero_fill_length)
+static void cached_deallocate(char* mem, uword_t zero_fill_length)
 {
     int line = 0;
     if (!cached_alloc[0]) {
@@ -185,15 +236,15 @@ static void cached_deallocate(char* mem, int zero_fill_length)
         int cached_size1 = usable_size(cached_alloc[1]);
         if (!(this_size > cached_size0 || this_size > cached_size1)) {
             // mem is not strictly larger than either cached block. Release it.
-            os_deallocate(mem - ALLOCATION_OVERHEAD,
-                          usable_size(mem) + ALLOCATION_OVERHEAD);
+            hopscotch_deallocate(mem - ALLOCATION_OVERHEAD,
+                                 usable_size(mem) + ALLOCATION_OVERHEAD);
             return;
         }
         // Evict and replace the smaller of the two cache entries.
         if (cached_size1 < cached_size0)
             line = 1;
-        os_deallocate(cached_alloc[line] - ALLOCATION_OVERHEAD,
-                      usable_size(cached_alloc[line]) + ALLOCATION_OVERHEAD);
+        hopscotch_deallocate(cached_alloc[line] - ALLOCATION_OVERHEAD,
+                             usable_size(cached_alloc[line]) + ALLOCATION_OVERHEAD);
     }
     memset(mem, 0, zero_fill_length);
     cached_alloc[line] = mem;
@@ -223,7 +274,7 @@ static void hopscotch_realloc(tableptr ht, int size, char hop_range)
     // The last few logical cells in the key array can use physical cells
     // at indices greater than 'size'; there's no wrapping back to index 0.
     int n_keys = size + (hop_range - 1);
-    unsigned storage_size = (sizeof (uword_t) + ht->value_size) * n_keys
+    uword_t storage_size = (sizeof (uword_t) + ht->value_size) * n_keys
         + sizeof (int) * size; // hop bitmasks
 
     if (ht->keys)
@@ -258,6 +309,7 @@ uword_t sxhash_simple_string(struct vector* string)
 #ifdef SIMPLE_CHARACTER_STRING_WIDETAG
     case SIMPLE_CHARACTER_STRING_WIDETAG:
         for(i=0;i<len;++i) MIX(char_string[i])
+        break;
 #endif
     case SIMPLE_BASE_STRING_WIDETAG:
         for(i=0;i<len;++i) MIX(base_string[i])
@@ -678,6 +730,44 @@ found0:
     return get_val(ht, index);
 }
 
+/* Return the address of the value associated with 'key',
+   insert 'key' with value 0 if it was not found. */
+void* hopscotch_get_ref(tableptr ht, uword_t key)
+{
+    gc_dcheck(key);
+    int index = hash(ht, key) & ht->mask;
+    unsigned bits = get_hop_mask(ht, index);
+    int __attribute__((unused)) probes = 0;
+    if (ht->compare) // Custom comparator
+        for ( ; bits ; bits >>= 1, ++index ) {
+            if ((bits & 1) && ht->compare(ht->keys[index], key))
+                goto found0;
+        }
+    else for ( ; bits ; bits >>= 4, index += 4)
+        if (bits & 0xf) {
+            probe(1, index+0, goto found0);
+            probe(2, index+1, goto found1);
+            probe(4, index+2, goto found2);
+            probe(8, index+3, goto found3);
+        }
+    tally_miss(ht, probes);
+    hopscotch_insert(ht, key, 0);
+    return hopscotch_get_ref(ht, key);
+found3: ++index;
+found2: ++index;
+found1: ++index;
+found0:
+    switch(ht->value_size) {
+#ifdef LISP_FEATURE_64_BIT
+    case 8: return (int64_t*)ht->values + index;
+#endif
+    case 4: return (int32_t*)ht->values + index;
+    case 2: return (int16_t*)ht->values + index;
+    case 1: return (int8_t *)ht->values + index;
+    }
+    return 0;
+}
+
 /* Update or insert a key/value pair. Return nonzero if
  * the key was inserted, or zero if the key existed. */
 int hopscotch_put(tableptr ht, uword_t key, sword_t val)
