@@ -33,10 +33,10 @@
  *   the value from interrupt_low_level_handlers[..], instead of the
  *   ordinary interrupt_handle_now(..) or interrupt_handle_later(..).
  *
- * o the SIGTRAP (Linux/Alpha) which Lisp code uses to handle breakpoints,
+ * o the SIGTRAP which Lisp code may uses to handle breakpoints,
  *   pseudo-atomic sections, and some classes of error (e.g. "function
  *   not defined").  This never goes anywhere near the Lisp handlers at all.
- *   See runtime/alpha-arch.c and code/signal.lisp
+ *   See src/code/signal.lisp
  *
  * - WHN 20000728, dan 20010128 */
 
@@ -112,7 +112,7 @@ boolean internal_errors_enabled = 0;
 static
 void (*interrupt_low_level_handlers[NSIG]) (int, siginfo_t*, os_context_t*);
 #endif
-union interrupt_handler interrupt_handlers[NSIG];
+lispobj lisp_sig_handlers[NSIG];
 
 /* Under Linux on some architectures, we appear to have to restore the
  * FPU control word from the context, as after the signal is delivered
@@ -171,56 +171,58 @@ static void sigmask_logandc(sigset_t *dest, const sigset_t *source)
 
 /* Foreign code may want to start some threads on its own.
  * Non-targetted, truly asynchronous signals can be delivered to
- * basically any thread, but invoking Lisp handlers in such foregign
+ * basically any thread, but invoking Lisp handlers in such foreign
  * threads is really bad, so let's resignal it.
  *
  * This should at least bring attention to the problem, but it cannot
  * work for SIGSEGV and similar. It is good enough for timers, and
  * maybe all deferrables. */
 
-#ifndef LISP_FEATURE_WIN32
-static void
-add_handled_signals(sigset_t *sigset)
-{
-    int i;
-    for(i = 1; i < NSIG; i++) {
-        if (!(ARE_SAME_HANDLER(interrupt_low_level_handlers[i], SIG_DFL)) ||
-            !(ARE_SAME_HANDLER(interrupt_handlers[i].c, SIG_DFL))) {
-            sigaddset(sigset, i);
-        }
-    }
-}
+#ifdef LISP_FEATURE_DARWIN
+pthread_key_t sigwait_bug_mitigation;
+int sigwait_bug_mitigation_count;
 #endif
 
-static boolean
-maybe_resignal_to_lisp_thread(int signal, os_context_t *context)
-{
-#ifndef LISP_FEATURE_WIN32
-    if (!lisp_thread_p(context)) {
-        if (!(sigismember(&deferrable_sigset,signal))) {
-            corruption_warning_and_maybe_lose
-#ifdef LISP_FEATURE_SB_THREAD
-                ("Received signal %d in non-lisp thread %lu, resignalling to a lisp thread.",
-                 signal, pthread_self());
+#ifdef LISP_FEATURE_WIN32
+#define resignal_to_lisp_thread(dummy1,dummy2) {}
 #else
-            ("Received signal %d in non-lisp thread, resignalling to a lisp thread.",
-             signal);
+static void
+resignal_to_lisp_thread(int signal, os_context_t *context)
+{
+#ifdef LISP_FEATURE_DARWIN
+    if (signal == SIG_STOP_FOR_GC && pthread_getspecific(sigwait_bug_mitigation)) {
+        // fprintf(stderr, "ignored a STOP_FOR_GC signal\n");
+        __sync_fetch_and_add(&sigwait_bug_mitigation_count, 1);
+        pthread_setspecific(sigwait_bug_mitigation, 0);
+        return;
+    }
 #endif
+    if (!sigismember(&deferrable_sigset,signal)) {
+        corruption_warning_and_maybe_lose
+#ifdef LISP_FEATURE_SB_THREAD
+            ("Received signal %d @ %lx in non-lisp"THREAD_ID_LABEL", resignaling to a lisp thread.",
+             signal, *os_context_pc_addr(context), THREAD_ID_VALUE);
+#else
+            ("Received signal %d in non-lisp thread, resignaling to a lisp thread.", signal);
+#endif
+    }
+    sigset_t sigset;
+    sigemptyset(&sigset);
+    int i;
+    for(i = 1; i < NSIG; i++) {
+        if (!ARE_SAME_HANDLER(interrupt_low_level_handlers[i], SIG_DFL)
+            || lisp_sig_handlers[i] != NIL) {
+            sigaddset(&sigset, i);
         }
-        sigset_t sigset;
-        sigemptyset(&sigset);
-        add_handled_signals(&sigset);
-        thread_sigmask(SIG_BLOCK, &sigset, 0);
-        // This arranges for every handled signal to be blocked on return from this
-        // handler invocation, presumably because that avoids further detours through
-        // this thread's handler for signals that we don't want it to handle.
-        sigmask_logior(os_context_sigmask_addr(context), &sigset);
-        kill(getpid(), signal);
-        return 1;
-    } else
-#endif
-        return 0;
+    }
+    thread_sigmask(SIG_BLOCK, &sigset, 0);
+    // This arranges for every handled signal to be blocked on return from this
+    // handler invocation, presumably because that avoids further detours through
+    // this thread's handler for signals that we don't want it to handle.
+    sigmask_logior(os_context_sigmask_addr(context), &sigset);
+    kill(getpid(), signal);
 }
+#endif
 
 #if INSTALL_SIG_MEMORY_FAULT_HANDLER && defined(THREAD_SANITIZER)
 /* Under TSAN, any delivered signal blocks all other signals regardless of the
@@ -253,7 +255,7 @@ maybe_resignal_to_lisp_thread(int signal, os_context_t *context)
  * low_level_handle_now_handler
  * low_level_maybe_now_maybe_later
  *
- * This gives us a single point of control (or six) over errno, fp
+ * This gives us a single point of control (or four) over errno, fp
  * control word, and fixing up signal context on sparc.
  *
  * The SPARC/Linux platform doesn't quite do signals the way we want
@@ -261,16 +263,20 @@ maybe_resignal_to_lisp_thread(int signal, os_context_t *context)
  * kernel properly, so we fix it up ourselves in the
  * arch_os_get_context(..) function. -- CSR, 2002-07-23
  */
+#ifdef LISP_FEATURE_WIN32
+# define should_handle_in_this_thread(c) (1)
+#else
+# define should_handle_in_this_thread(c) lisp_thread_p(c)
+#endif
 #define SAVE_ERRNO(signal,context,void_context)                 \
     {                                                           \
         int _saved_errno = errno;                               \
         UNBLOCK_SIGSEGV();                                      \
         RESTORE_FP_CONTROL_WORD(context,void_context);          \
-        if (!maybe_resignal_to_lisp_thread(signal, context))    \
-        {
+        if (should_handle_in_this_thread(context)) {
 
 #define RESTORE_ERRNO                                           \
-        }                                                       \
+        } else resignal_to_lisp_thread(signal,void_context);    \
         errno = _saved_errno;                                   \
     }
 
@@ -363,15 +369,23 @@ sigaddset_deferrable(sigset_t *s)
 #else
     sigaddset(s, SIGPOLL);
 #endif
-#ifndef LISP_FEATURE_HPUX
+#ifndef LISP_FEATURE_LIBUNWIND_BACKTRACE
     sigaddset(s, SIGXCPU);
-    sigaddset(s, SIGXFSZ);
 #endif
+    sigaddset(s, SIGXFSZ);
     sigaddset(s, SIGVTALRM);
     sigaddset(s, SIGPROF);
     sigaddset(s, SIGWINCH);
 }
 
+static void
+sigaddset_gc(sigset_t __attribute__((unused)) *sigset)
+{
+#ifdef THREADS_USING_GCSIGNAL
+    sigaddset(sigset,SIG_STOP_FOR_GC);
+#endif
+}
+
 void
 sigaddset_blockable(sigset_t *sigset)
 {
@@ -379,14 +393,6 @@ sigaddset_blockable(sigset_t *sigset)
     sigaddset_gc(sigset);
 }
 
-void
-sigaddset_gc(sigset_t __attribute__((unused)) *sigset)
-{
-#ifdef THREADS_USING_GCSIGNAL
-    sigaddset(sigset,SIG_STOP_FOR_GC);
-#endif
-}
-
 /* initialized in interrupt_init */
 sigset_t deferrable_sigset;
 sigset_t blockable_sigset;
@@ -1139,7 +1145,9 @@ void
 interrupt_handle_now(int signal, siginfo_t *info, os_context_t *context)
 {
     boolean were_in_lisp;
-    union interrupt_handler handler;
+    lispobj handler = lisp_sig_handlers[signal];
+
+    if (!functionp(handler)) return;
 
     check_blockables_blocked_or_lose(0);
 
@@ -1148,79 +1156,42 @@ interrupt_handle_now(int signal, siginfo_t *info, os_context_t *context)
         check_interrupts_enabled_or_lose(context);
 #endif
 
-    handler = interrupt_handlers[signal];
-
-    if (ARE_SAME_HANDLER(handler.c, SIG_IGN)) {
-        return;
-    }
-
     were_in_lisp = !foreign_function_call_active_p(arch_os_get_current_thread());
     if (were_in_lisp)
     {
         fake_foreign_function_call(context);
     }
 
-    FSHOW_SIGNAL((stderr,
-                  "/entering interrupt_handle_now(%d, info, context)\n",
-                  signal));
-
-    if (ARE_SAME_HANDLER(handler.c, SIG_DFL)) {
-
-        /* This can happen if someone tries to ignore or default one
-         * of the signals we need for runtime support, and the runtime
-         * support decides to pass on it. */
-        lose("no handler for signal %d in interrupt_handle_now(..)", signal);
-
-        // BUG: if a C function pointer can be misaligned such that it
-        // looks to satisfy functionp() then we do the wrong thing.
-    } else if (functionp(handler.lisp)) {
         /* Once we've decided what to do about contexts in a
          * return-elsewhere world (the original context will no longer
          * be available; should we copy it or was nobody using it anyway?)
          * then we should convert this to return-elsewhere */
 
-#ifndef LISP_FEATURE_SB_SAFEPOINT
+#if !defined(LISP_FEATURE_SB_SAFEPOINT) && defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
         /* Leave deferrable signals blocked, the handler itself will
          * allow signals again when it sees fit. */
-#ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
         /* handler.lisp will hide from the GC, will be enabled in the handler itself.
          * Not a problem for the conservative GC. */
         unblock_gc_signals();
 #endif
-#else
+
         WITH_GC_AT_SAFEPOINTS_ONLY()
-#endif
         { // the block is needed for WITH_GC_AT_SAFEPOINTS_ONLY() to work
             DX_ALLOC_SAP(context_sap, context);
             DX_ALLOC_SAP(info_sap, info);
 
             FSHOW_SIGNAL((stderr,"/calling Lisp-level handler\n"));
 
-            funcall3(handler.lisp,
+            funcall3(handler,
                      make_fixnum(signal),
                      info_sap,
                      context_sap);
         }
-    } else {
-        /* This cannot happen in sane circumstances. */
-
-        FSHOW_SIGNAL((stderr,"/calling C-level handler\n"));
-
-#if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
-        /* Allow signals again. */
-        thread_sigmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
-        (*handler.c)(signal, info, context);
-#endif
-    }
 
     if (were_in_lisp)
     {
         undo_fake_foreign_function_call(context); /* block signals again */
     }
-
-    FSHOW_SIGNAL((stderr,
-                  "/returning from interrupt_handle_now(%d, info, context)\n",
-                  signal));
 }
 
 /* This is called at the end of a critical section if the indications
@@ -1244,9 +1215,9 @@ run_deferred_handler(struct interrupt_data *data, os_context_t *context)
 }
 
 #ifndef LISP_FEATURE_WIN32
-boolean
-maybe_defer_handler(void *handler, struct interrupt_data *data,
-                    int signal, siginfo_t *info, os_context_t *context)
+static boolean
+can_handle_now(void *handler, struct interrupt_data *data,
+               int signal, siginfo_t *info, os_context_t *context)
 {
     struct thread *thread=arch_os_get_current_thread();
 
@@ -1257,7 +1228,7 @@ maybe_defer_handler(void *handler, struct interrupt_data *data,
     if (thread->interrupt_data->pending_handler)
         lose("there is a pending handler already (PA)");
     if (data->gc_blocked_deferrables)
-        lose("maybe_defer_handler: gc_blocked_deferrables true");
+        lose("can_handle_now: gc_blocked_deferrables true");
 
     /* If interrupts are disabled then INTERRUPT_PENDING is set and
      * not PSEDUO_ATOMIC_INTERRUPTED. This is important for a pseudo
@@ -1270,33 +1241,33 @@ maybe_defer_handler(void *handler, struct interrupt_data *data,
     if ((read_TLS(INTERRUPTS_ENABLED,thread) == NIL) ||
         in_leaving_without_gcing_race_p(thread)) {
         FSHOW_SIGNAL((stderr,
-                      "/maybe_defer_handler(%p,%d): deferred (RACE=%d)\n",
+                      "/can_handle_now(%p,%d): deferred (RACE=%d)\n",
                       handler,signal,
                       in_leaving_without_gcing_race_p(thread)));
         store_signal_data_for_later(data,handler,signal,info,context);
         write_TLS(INTERRUPT_PENDING, T,thread);
         check_interrupt_context_or_lose(context);
-        return 1;
+        return 0;
     }
     /* a slightly confusing test. arch_pseudo_atomic_atomic() doesn't
      * actually use its argument for anything on x86, so this branch
      * may succeed even when context is null (gencgc alloc()) */
     if (arch_pseudo_atomic_atomic(context)) {
         FSHOW_SIGNAL((stderr,
-                      "/maybe_defer_handler(%p,%d): deferred(PA)\n",
+                      "/can_handle_now(%p,%d): deferred(PA)\n",
                       handler,signal));
         store_signal_data_for_later(data,handler,signal,info,context);
         arch_set_pseudo_atomic_interrupted(context);
         check_interrupt_context_or_lose(context);
-        return 1;
+        return 0;
     }
 
     check_interrupt_context_or_lose(context);
 
     FSHOW_SIGNAL((stderr,
-                  "/maybe_defer_handler(%p,%d): not deferred\n",
+                  "/can_handle_now(%p,%d): not deferred\n",
                   handler,signal));
-    return 0;
+    return 1;
 }
 
 static void
@@ -1335,7 +1306,7 @@ maybe_now_maybe_later(int signal, siginfo_t *info, void *void_context)
     SAVE_ERRNO(signal,context,void_context);
     struct thread *thread = arch_os_get_current_thread();
     struct interrupt_data *data = thread->interrupt_data;
-    if(!maybe_defer_handler(interrupt_handle_now,data,signal,info,context))
+    if (can_handle_now(interrupt_handle_now, data, signal, info, context))
         interrupt_handle_now(signal, info, context);
     RESTORE_ERRNO;
 }
@@ -1357,8 +1328,7 @@ low_level_maybe_now_maybe_later(int signal, siginfo_t *info, void *void_context)
     struct thread *thread = arch_os_get_current_thread();
     struct interrupt_data *data = thread->interrupt_data;
 
-    if(!maybe_defer_handler(low_level_interrupt_handle_now,data,
-                            signal,info,context))
+    if (can_handle_now(low_level_interrupt_handle_now, data, signal, info, context))
         low_level_interrupt_handle_now(signal, info, context);
     RESTORE_ERRNO;
 }
@@ -1429,7 +1399,7 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
              (lispobj)fixnum_value(thread->state));
     }
 
-    set_thread_state(thread,STATE_STOPPED);
+    set_thread_state(thread, STATE_STOPPED, 0);
     FSHOW_SIGNAL((stderr,"suspended\n"));
 
     /* While waiting for gc to finish occupy ourselves with zeroing
@@ -1842,7 +1812,7 @@ extern void restore_sbcl_signals () {
     for (signal = 0; signal < NSIG; signal++) {
         interrupt_handler_t handler = interrupt_low_level_handlers[signal];
         if (handler) {
-            undoably_install_low_level_interrupt_handler(signal, handler);
+            ll_install_handler(signal, handler);
         }
     }
 }
@@ -1913,6 +1883,9 @@ spawn_signal_thread_handler(int signal, siginfo_t *info, void *void_context)
     pthread_attr_t attr;
     pthread_t th;
 
+    // This is total bullcrap. None of these functions are specified
+    // as being async-signal-safe, yet here were are in a signal handler
+    // trying to create a thread.
     if (pthread_attr_init(&attr))
         goto lost;
     if (pthread_attr_setstacksize(&attr, thread_control_stack_size))
@@ -1939,7 +1912,7 @@ low_level_handle_now_handler(int signal, siginfo_t *info, void *void_context)
 }
 
 void
-undoably_install_low_level_interrupt_handler (int signal,
+ll_install_handler (int signal,
                                               interrupt_handler_t handler)
 {
     struct sigaction sa;
@@ -1969,7 +1942,7 @@ undoably_install_low_level_interrupt_handler (int signal,
 #if defined(LISP_FEATURE_C_STACK_IS_CONTROL_STACK)
     if(signal==SIG_MEMORY_FAULT) {
         sa.sa_flags |= SA_ONSTACK;
-# ifdef LISP_FEATURE_SB_SAFEPOINT
+# if defined LISP_FEATURE_SB_SAFEPOINT && !defined LISP_FEATURE_DARWIN
         sigaddset(&sa.sa_mask, SIGRTMIN);
         sigaddset(&sa.sa_mask, SIGRTMIN+1);
 # endif
@@ -1983,76 +1956,51 @@ undoably_install_low_level_interrupt_handler (int signal,
 #endif
 
 /* This is called from Lisp. */
-uword_t
-install_handler(int signal, void handler(int, siginfo_t*, os_context_t*),
-                lispobj ohandler, // (SIMPLE-VECTOR 1) as a tagged pointer
+void
+install_handler(int signal, lispobj handler,
                 int __attribute__((unused)) synchronous)
 {
 #ifndef LISP_FEATURE_WIN32
     struct sigaction sa;
-    sigset_t old;
 
-    FSHOW((stderr, "/entering POSIX install_handler(%d, ..)\n", signal));
-
-    block_blockable_signals(&old);
-
-    FSHOW((stderr, "/interrupt_low_level_handlers[signal]=%p\n",
-           interrupt_low_level_handlers[signal]));
     if (interrupt_low_level_handlers[signal]==0) {
-        if (handler == 0)
-            sa.sa_sigaction = (void (*)(int, siginfo_t*, void*))SIG_DFL;
-        else if ((lispobj)handler == 1)
-            sa.sa_sigaction = (void (*)(int, siginfo_t*, void*))SIG_IGN;
+        // Our "abstract" values for SIG_DFL and SIG_IGN are 0 and 1
+        // respectively which are probably the real values from signal.h
+        // but this way way don't need to put them in grovel-headers.c
+        if (fixnump(handler)) {
+            memset(&sa, 0, sizeof sa);
+            sa.sa_handler = handler ? SIG_IGN : SIG_DFL;
+            // assign the OS level action before clearing the lisp function.
+            // (If a signal were to be delivered to the C trampoline when the lisp
+            // function is NIL, we'd get the effect of :IGNORE regardless
+            // of what the default action should be)
+            sigaction(signal, &sa, NULL);
+            lisp_sig_handlers[signal] = NIL;
+            return;
+        }
 #ifdef LISP_FEATURE_SB_SAFEPOINT_STRICTLY
-        else if (signal == SIGPROF)
+        if (signal == SIGPROF)
             sa.sa_sigaction = sigprof_handler_trampoline;
         else if (!synchronous)
             sa.sa_sigaction = spawn_signal_thread_handler;
+        else
 #endif
-        else if (sigismember(&deferrable_sigset, signal))
+        if (sigismember(&deferrable_sigset, signal))
             sa.sa_sigaction = maybe_now_maybe_later;
         else
             sa.sa_sigaction = interrupt_handle_now_handler;
 
         sa.sa_mask = blockable_sigset;
         sa.sa_flags = SA_SIGINFO | SA_RESTART | OS_SA_NODEFER;
+        // ensure the C handler sees a lisp function before doing sigaction()
+        lisp_sig_handlers[signal] = handler;
         sigaction(signal, &sa, NULL);
+    } else {
+        // When there's a low-level handler, we must leave it alone.
+        // Give it the lisp function to call if it decides to forward a signal.
+        // SIG_IGN and SIG_DFL don't always do what you think in such case.
+        lisp_sig_handlers[signal] = functionp(handler) ? handler : NIL;
     }
-
-    // We can GC-safely read interrupt_low_level_handlers[signal] and assign
-    // into ohandler despite that the C stack and registers are not roots on
-    // precise GC. That is, if GC were to occur as depicted:
-    //   obj = old_handler
-    //   -- GC signal occurs here --
-    //   result = obj
-    // then the register or stack word holding 'obj' could be obsolete.
-    // But that won't happen- the STOP_FOR_GC signal is currently blocked
-    // due to the block_blockable_signals() at the top of this function.
-    // However, there is an unlikely other reason for a wrong result -
-    // if the C function address does not have at least N_FIXNUM_TAG_BITS
-    // of alignment, then we say that it was NIL instead of a fixnum.
-    lispobj obj = interrupt_handlers[signal].lisp;
-    uword_t result = -1;
-    if ((void*)obj == (void*)SIG_DFL)
-        result = 0;
-    else if ((void*)obj == (void*)SIG_IGN)
-        result = 1;
-    else if (ohandler) // only store into 'ohandler' when it was supplied
-        VECTOR(ohandler)->data[0] = (functionp(obj)||fixnump(obj)) ? obj : NIL;
-
-    interrupt_handlers[signal].c = handler;
-
-    thread_sigmask(SIG_SETMASK, &old, 0);
-
-    FSHOW((stderr, "/leaving POSIX install_handler(%d, ..)\n", signal));
-
-    // After the thread's signal mask is restored to the mask in 'old',
-    // it is not possible to GC-safely refer to a Lisp function from C code,
-    // which is to say, naively utilizing "return obj;" would be wrong.
-    return result;
-#else
-    /* Probably-wrong Win32 hack */
-    return 0;
 #endif
 }
 
@@ -2073,7 +2021,7 @@ void
 interrupt_init(void)
 {
 #if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
-    int i;
+    int __attribute__((unused)) i;
     SHOW("entering interrupt_init()");
     sigemptyset(&deferrable_sigset);
     sigemptyset(&blockable_sigset);
@@ -2083,18 +2031,18 @@ interrupt_init(void)
     sigaddset_gc(&gc_sigset);
 #endif
 
+#ifdef LISP_FEATURE_LIBUNWIND_BACKTRACE
+    // Use this only if you know what you're doing
+    void backtrace_lisp_threads(int, siginfo_t*, os_context_t*);
+    ll_install_handler(SIGXCPU, backtrace_lisp_threads);
+#endif
+
 #ifndef LISP_FEATURE_WIN32
     /* Set up high level handler information. */
     for (i = 0; i < NSIG; i++) {
-        interrupt_handlers[i].c =
-            /* (The cast here blasts away the distinction between
-             * SA_SIGACTION-style three-argument handlers and
-             * signal(..)-style one-argument handlers, which is OK
-             * because it works to call the 1-argument form where the
-             * 3-argument form is expected.) */
-            (void (*)(int, siginfo_t*, os_context_t*))SIG_DFL;
+        lisp_sig_handlers[i] = NIL;
     }
-    undoably_install_low_level_interrupt_handler(SIGABRT, sigabrt_handler);
+    ll_install_handler(SIGABRT, sigabrt_handler);
 #endif
     SHOW("returning from interrupt_init()");
 }
@@ -2119,15 +2067,15 @@ lisp_memory_fault_error(os_context_t *context, os_vm_address_t addr)
     unsigned int offset = code ? pc - (char*)code : 0;
     if (offset)
         corruption_warning_and_maybe_lose(
-            "Memory fault at %p (pc=%p [code %p+0x%X ID 0x%x], fp=%p, sp=%p) tid %#lx",
+            "Memory fault at %p (pc=%p [code %p+0x%X ID 0x%x], fp=%p, sp=%p)" THREAD_ID_LABEL,
             addr, pc, code, offset, code_serialno(code),
             os_context_frame_pointer(context),
-            *os_context_sp_addr(context), thread_self()); // = 0 if -sb-thread
+            *os_context_sp_addr(context), THREAD_ID_VALUE);
     else
         corruption_warning_and_maybe_lose(
-            "Memory fault at %p (pc=%p, fp=%p, sp=%p) tid %#lx",
+            "Memory fault at %p (pc=%p, fp=%p, sp=%p)" THREAD_ID_LABEL,
             addr, pc, os_context_frame_pointer(context),
-            *os_context_sp_addr(context), thread_self()); // = 0 if -sb-thread
+            *os_context_sp_addr(context), THREAD_ID_VALUE);
 #else
     corruption_warning_and_maybe_lose("Memory fault at %p (pc=%p)",
                                       addr, *os_context_pc_addr(context));
