@@ -199,8 +199,8 @@ dump_cmd(char **ptr)
                     && valid_widetag_p(header_widetag(word))) {
                     printf(" %s", widetag_names[header_widetag(word)>>2]);
                     next_object += sizetab[header_widetag(word)](next_object);
-                } else if (is_cons_half(word)) {
-                    next_object += 2;
+                } else if (!is_header(word)) {
+                    next_object += CONS_SIZE;
                 } else { // disable decoder if weirdness observed
                     decode = 0;
                 }
@@ -492,18 +492,17 @@ catchers_cmd(char __attribute__((unused)) **ptr)
         printf("There are no active catchers!\n");
     else {
         while (catch != NULL) {
-            printf("0x%08lX:\n\tuwp: 0x%08lX\n\tfp: 0x%08lX\n\t"
-                   "code: 0x%08lX\n\tentry: 0x%08lX\n\ttag: ",
-                   (long unsigned)catch,
-                   (long unsigned)(catch->uwp),
-                   (long unsigned)(catch->cfp),
+            printf("%p:\n\tuwp  : %p\n\tfp   : %p\n\t"
+                   "code : %p\n\tentry: %p\n\ttag: ",
+                   catch,
+                   catch->uwp,
+                   catch->cfp,
 #if defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64)
-                   (long unsigned)component_ptr_from_pc((void*)catch->entry_pc)
-                       + OTHER_POINTER_LOWTAG,
+                   component_ptr_from_pc((void*)catch->entry_pc),
 #else
-                   (long unsigned)(catch->code),
+                   catch->code,
 #endif
-                   (long unsigned)(catch->entry_pc));
+                   (void*)(catch->entry_pc));
             brief_print((lispobj)catch->tag);
             catch = catch->previous_catch;
         }
@@ -511,13 +510,31 @@ catchers_cmd(char __attribute__((unused)) **ptr)
     return 0;
 }
 
+/* SIGINT handler that invokes the monitor (for when Lisp isn't up to it) */
+static void
+sigint_handler(int __attribute__((unused)) signal,
+               siginfo_t __attribute__((unused)) *info,
+               void *context)
+{
+    extern void ldb_monitor();
+    fprintf(stderr, "\nSIGINT hit at %p\n", (void*)*os_context_pc_addr(context));
+    ldb_monitor();
+    fprintf(stderr, "Returning to lisp (if you're lucky).\n");
+}
+
 static int
 grab_sigs_cmd(char __attribute__((unused)) **ptr)
 {
-    extern void sigint_init(void);
-
-    printf("Grabbing signals.\n");
-    sigint_init();
+#ifdef LISP_FEATURE_WIN32
+    fprintf(stderr, "sorry no can do\n"); fflush(stderr);
+#else
+    printf("Grabbing SIGINT.\n");
+    struct sigaction sa;
+    sigemptyset(&sa.sa_mask);
+    sa.sa_flags = SA_SIGINFO;
+    sa.sa_sigaction = sigint_handler;
+    sigaction(SIGINT, &sa, 0);
+#endif
     return 0;
 }
 
