@@ -152,8 +152,6 @@ vodxprint_fun(const char *fmt, va_list args)
 #endif
     int original_errno = errno;
 
-    QSHOW_BLOCK;
-
     char buf[1024];
     int n = 0;
 
@@ -184,8 +182,6 @@ vodxprint_fun(const char *fmt, va_list args)
     fflush(stderr);
 #endif
 
-    QSHOW_UNBLOCK;
-
 #ifdef LISP_FEATURE_WIN32
     SetLastError(lastError);
 #endif
@@ -647,6 +643,14 @@ static void print_fun_or_otherptr(lispobj obj)
         print_slots(symbol_slots, count & 0xFF, ptr);
         if (symbol_function(ptr-1) != NIL)
             print_obj("fun: ", symbol_function(ptr-1));
+#ifdef LISP_FEATURE_SB_THREAD
+        int tlsindex = tls_index_of((struct symbol*)(ptr-1));
+        struct thread*th = arch_os_get_current_thread();
+        if (th != 0 && tlsindex != 0) {
+            lispobj v = *(lispobj*)(tlsindex + (char*)th);
+            print_obj("tlsval: ", v);
+        }
+#endif
         break;
 
 #if N_WORD_BITS == 32
@@ -774,11 +778,7 @@ static void print_fun_or_otherptr(lispobj obj)
 
     case SAP_WIDETAG:
         NEWLINE_OR_RETURN;
-#ifndef LISP_FEATURE_ALPHA
-        printf("0x%08lx", (unsigned long) *ptr);
-#else
-        printf("0x%016lx", *(lispobj*)(ptr+1));
-#endif
+        printf("%p", (void*)*ptr);
         break;
 
     case WEAK_POINTER_WIDETAG:
@@ -921,10 +921,7 @@ struct vector * symbol_name(lispobj * sym)
     sym = native_pointer(forwarding_pointer_value(sym));
   if (lowtag_of(((struct symbol*)sym)->name) != OTHER_POINTER_LOWTAG)
       return NULL;
-  lispobj * name = native_pointer(((struct symbol*)sym)->name);
-  if (forwarding_pointer_p(name))
-      name = native_pointer(forwarding_pointer_value(name));
-  return (struct vector*)name;
+  return VECTOR(follow_maybe_fp(((struct symbol*)sym)->name));
 }
 struct vector * classoid_name(lispobj * classoid)
 {
