@@ -64,41 +64,10 @@
 #include "genesis/static-symbols.h"
 #include "genesis/symbol.h"
 
-
-#ifdef irix
-#include <string.h>
-#include "interr.h"
-#endif
-
 static char libpath[] = "../lib/sbcl";
-char *runtime_home;
+char *sbcl_runtime_home;
+char *sbcl_runtime;
 
-#ifdef LISP_FEATURE_HPUX
-extern void *return_from_lisp_stub;
-#include "genesis/closure.h"
-#include "genesis/simple-fun.h"
-#endif
-
-
-/* SIGINT handler that invokes the monitor (for when Lisp isn't up to it) */
-static void
-sigint_handler(int __attribute__((unused)) signal,
-               siginfo_t __attribute__((unused)) *info,
-               os_context_t *context)
-{
-    lose("\nSIGINT hit at 0x%08lX",
-         (unsigned long) *os_context_pc_addr(context));
-}
-
-/* (This is not static, because we want to be able to call it from
- * Lisp land.) */
-void
-sigint_init(void)
-{
-    SHOW("entering sigint_init()");
-    install_handler(SIGINT, sigint_handler, 0, 1);
-    SHOW("leaving sigint_init()");
-}
 
 /*
  * helper functions for dealing with command line args
@@ -236,7 +205,7 @@ search_for_core ()
 
     if (!(env_sbcl_home && *env_sbcl_home) ||
         stat(env_sbcl_home, &filename_stat))
-        env_sbcl_home = runtime_home;
+        env_sbcl_home = sbcl_runtime_home;
     lookhere = (char *) calloc(strlen(env_sbcl_home) +
                                strlen(libpath) +
                                strlen(stem) +
@@ -260,7 +229,7 @@ search_for_core ()
             if (core) {
                 free(lookhere);
             } else {
-                lose("Can't find core file relative to %s", env_sbcl_home);
+                return NULL;
             }
         }
     }
@@ -372,7 +341,6 @@ parse_size_arg(char *arg, char *arg_name)
 char **posix_argv;
 char *core_string;
 
-char *saved_runtime_path = NULL;
 #if defined(LISP_FEATURE_WIN32) && defined(LISP_FEATURE_SB_THREAD)
 void pthreads_win32_init();
 #endif
@@ -392,6 +360,33 @@ static void print_environment(int argc, char *argv[])
         ++n;
     }
 }
+
+char * sb_realpath (char *);
+char *dir_name(char *path) {
+    if (path == NULL)
+        return NULL;
+
+    char* result;
+    char slashchar =
+#ifdef LISP_FEATURE_WIN32
+        '\\';
+#else
+    '/';
+#endif
+
+    char *slash = strrchr(path, slashchar);
+
+    if (slash) {
+        int prefixlen = slash - path + 1; // keep the slash in the prefix
+        result = successful_malloc(prefixlen + 1);
+        memcpy(result, path, prefixlen);
+        result[prefixlen] = 0;
+        return result;
+    } else {
+        return NULL;
+    }
+}
+
 
 extern void write_protect_immobile_space();
 struct lisp_startup_options lisp_startup_options;
@@ -408,7 +403,6 @@ sbcl_main(int argc, char *argv[], char *envp[])
     char *core = 0;
     char **sbcl_argv = 0;
     os_vm_offset_t embedded_core_offset = 0;
-    char *runtime_path = 0;
 
     /* other command line options */
     boolean end_runtime_options = 0;
@@ -427,65 +421,25 @@ sbcl_main(int argc, char *argv[], char *envp[])
     memsize_options.present_in_core = 0;
 
     boolean have_hardwired_spaces = os_preinit(argv, envp);
-#if defined(LISP_FEATURE_WIN32) && defined(LISP_FEATURE_SB_THREAD)
-    pthreads_win32_init();
-#endif
 
     interrupt_init();
     block_blockable_signals(0);
 
-    /* Save the argv[0] derived runtime path in case
-     * os_get_runtime_executable_path(1) isn't able to get an
-     * externally-usable path later on. */
-    saved_runtime_path = search_for_executable(argv[0]);
-
     /* Check early to see if this executable has an embedded core,
      * which also populates runtime_options if the core has runtime
      * options */
-    runtime_path = os_get_runtime_executable_path(0);
+    if (!(sbcl_runtime = os_get_runtime_executable_path()))
+        sbcl_runtime = search_for_executable(argv[0]);
 
-    /* Set 'runtime_home' to
-     * "<here>" based on how this executable was invoked. */
-    {
-        char *exename = argv[0]; // Use as-it, not truenameified
-        char slashchar =
-#ifdef LISP_FEATURE_WIN32
-          '\\';
-#else
-          '/';
-#endif
+    if (!(sbcl_runtime_home = dir_name(argv[0])))
+      if (!(sbcl_runtime_home = dir_name(sbcl_runtime)))
+        sbcl_runtime_home = libpath;
 
-        char *slash = strrchr(exename, slashchar);
-        if (!slash) {
-            if (runtime_path) {
-                exename = runtime_path;
-            } else if (saved_runtime_path) {
-                exename = saved_runtime_path;
-            }
-            slash = strrchr(exename, slashchar);
-        }
-
-        if (!slash) {
-            runtime_home = libpath;
-        } else {
-            int prefixlen = slash - exename + 1; // keep the slash in the prefix
-            runtime_home = successful_malloc(prefixlen + 1);
-            memcpy(runtime_home, exename, prefixlen);
-            runtime_home[prefixlen] = 0;
-
-        }
-    }
-
-    if (runtime_path || saved_runtime_path) {
-        os_vm_offset_t offset = search_for_embedded_core(
-            runtime_path ? runtime_path : saved_runtime_path,
-            &memsize_options);
+    if (sbcl_runtime) {
+        os_vm_offset_t offset = search_for_embedded_core(sbcl_runtime, &memsize_options);
         if (offset != -1) {
             embedded_core_offset = offset;
-            core = (runtime_path ? runtime_path :
-                    copied_string(saved_runtime_path));
-        } else if (runtime_path) {
-            free(runtime_path);
+            core = sbcl_runtime;
         }
     }
 
@@ -649,8 +603,23 @@ sbcl_main(int argc, char *argv[], char *envp[])
     gc_init();
 
     /* If no core file was specified, look for one. */
-    if (!core) {
-        core = search_for_core();
+    if (!core && !(core = search_for_core())) {
+      /* Try resolving symlinks */
+      if (sbcl_runtime) {
+        free(sbcl_runtime_home);
+        char* real = sb_realpath(sbcl_runtime);
+        if (!real)
+          goto lose;
+        sbcl_runtime_home = dir_name(real);
+        free(real);
+        if (!sbcl_runtime_home)
+          goto lose;
+        if(!(core = search_for_core()))
+          goto lose;
+      } else {
+      lose:
+        lose("Can't find sbcl.core");
+      }
     }
 
     if (embedded_core_offset)
@@ -703,13 +672,6 @@ sbcl_main(int argc, char *argv[], char *envp[])
      * since it was too soon earlier to handle write faults. */
     write_protect_immobile_space();
 #endif
-#ifdef LISP_FEATURE_HPUX
-    // FIXME: obvious bitrot here. 23 isn't the offset to anything.
-    /* -1 = CLOSURE_FUN_OFFSET, 23 = SIMPLE_FUN_CODE_OFFSET, we are
-     * not in __ASSEMBLER__ so we cant reach them. */
-    return_from_lisp_stub = (void *) ((char *)*((unsigned long *)
-                 ((char *)initial_function + -1)) + 23);
-#endif
 
     arch_install_interrupt_handlers();
 #ifndef LISP_FEATURE_WIN32
@@ -727,7 +689,7 @@ sbcl_main(int argc, char *argv[], char *envp[])
 
     FSHOW((stderr, "/funcalling initial_function=0x%lx\n",
           (unsigned long)initial_function));
-    create_initial_thread(initial_function);
+    create_main_lisp_thread(initial_function);
     lose("unexpected return from initial thread in main()");
     return 0;
 }
