@@ -60,22 +60,9 @@
 #include "thread.h"
 #include "align.h"
 
-#ifndef LISP_FEATURE_SB_THREAD
-/* dummy definition to reduce ifdef clutter */
-#define WITH_GC_AT_SAFEPOINTS_ONLY() if (0) ; else
-#endif
-
-os_vm_size_t os_vm_page_size;
-
 #include "gc.h"
 #include "gencgc-internal.h"
 #include <wincrypt.h>
-
-#if 0
-int linux_sparc_siginfo_bug = 0;
-int linux_supports_futex=0;
-#endif
-
 #include <stdarg.h>
 #include <string.h>
 
@@ -317,7 +304,7 @@ void unmap_gc_page()
 
 #endif
 
-#if defined(LISP_FEATURE_SB_DYNAMIC_CORE)
+#if defined(LISP_FEATURE_LINKAGE_TABLE)
 /* This feature has already saved me more development time than it
  * took to implement.  In its current state, ``dynamic RT<->core
  * linking'' is a protocol of initialization of C runtime and Lisp
@@ -366,7 +353,7 @@ void unmap_gc_page()
  * bundle'' that rolls up your patch, redumps and -- presto -- 100MiB
  * program is fixed by sending and loading a 50KiB thingie.
  *
- * However, until LISP_FEATURE_SB_DYNAMIC_CORE, if your bug were fixed
+ * However, until LISP_FEATURE_LINKAGE_TABLE, if your bug were fixed
  * by modifying two lines of _C_ sources, a customer described above
  * had to be ready to receive and reinstall a new 100MiB
  * executable. With the aid of code below, deploying such a fix
@@ -401,7 +388,7 @@ void unmap_gc_page()
  * to leave old-style linking code in place for the sake of
  * _non-linkage-table_ platforms (they probably don't have -ldl or its
  * equivalent, like LL/GPA, at all) -- but i did it usually by moving
- * the entire `old style' code under #-sb-dynamic-core and
+ * the entire `old style' code under #-linkage-table and
  * refactoring the `new style' branch, instead of cutting the tail
  * piecemeal and increasing #+-ifdeffery amount & the world enthropy.
  *
@@ -556,7 +543,7 @@ u32 os_get_build_time_shared_libraries(u32 excl_maximum,
                    Anyway: using a module handle more than once will
                    do no harm, but it slows down the startup (even
                    now, our startup time is not a pleasant topic to
-                   discuss when it comes to :sb-dynamic-core; there is
+                   discuss when it comes to :linkage-table; there is
                    an obvious direction to go for speed, though --
                    instead of resolving symbols one-by-one, locate PE
                    export directories -- they are sorted by symbol
@@ -611,7 +598,7 @@ void* os_dlsym_default(char* name)
     return result;
 }
 
-#endif /* SB_DYNAMIC_CORE */
+#endif /* LINKAGE_TABLE */
 
 #if defined(LISP_FEATURE_SB_THREAD)
 /* We want to get a slot in TIB that (1) is available at constant
@@ -655,6 +642,7 @@ void* os_dlsym_default(char* name)
 
 int os_preinit(char *argv[], char *envp[])
 {
+    pthreads_win32_init();
 #ifdef LISP_FEATURE_X86
     DWORD slots[TLS_MINIMUM_AVAILABLE];
     DWORD key;
@@ -684,64 +672,14 @@ int os_preinit(char *argv[], char *envp[])
 }
 #endif  /* LISP_FEATURE_SB_THREAD */
 
-
-#ifdef LISP_FEATURE_X86_64
-/* Windows has 32-bit 'longs', so printf...%lX (and other %l patterns) doesn't
- * work well with address-sized values, like it's done all over the place in
- * SBCL. And msvcrt uses I64, not LL, for printing long longs.
- *
- * I've already had enough search/replace with longs/words/intptr_t for today,
- * so I prefer to solve this problem with a format string translator. */
-
-/* There is (will be) defines for printf and friends. */
-
-static int translating_vfprintf(FILE*stream, const char *fmt, va_list args)
-{
-    char translated[1024];
-    unsigned i=0, delta = 0;
-
-    while (fmt[i-delta] && i<sizeof(translated)-1) {
-        if((fmt[i-delta]=='%')&&
-           (fmt[i-delta+1]=='l')) {
-            translated[i++]='%';
-            translated[i++]='I';
-            translated[i++]='6';
-            translated[i++]='4';
-            delta += 2;
-        } else {
-            translated[i]=fmt[i-delta];
-            ++i;
-        }
-    }
-    translated[i++]=0;
-    return vfprintf(stream,translated,args);
-}
-
-int printf(const char*fmt,...)
-{
-    va_list args;
-    va_start(args,fmt);
-    return translating_vfprintf(stdout,fmt,args);
-}
-int fprintf(FILE*stream,const char*fmt,...)
-{
-    va_list args;
-    va_start(args,fmt);
-    return translating_vfprintf(stream,fmt,args);
-}
-
-#endif
-
 int os_number_of_processors = 1;
 
-BOOL WINAPI CancelIoEx(HANDLE handle, LPOVERLAPPED overlapped);
-typeof(CancelIoEx) *ptr_CancelIoEx;
-BOOL WINAPI CancelSynchronousIo(HANDLE threadHandle);
-typeof(CancelSynchronousIo) *ptr_CancelSynchronousIo;
+BOOL (*ptr_CancelIoEx)(HANDLE /*handle*/, LPOVERLAPPED /*overlapped*/);
+BOOL (*ptr_CancelSynchronousIo)(HANDLE /*threadHandle*/);
 
 #define RESOLVE(hmodule,fn)                     \
     do {                                        \
-        ptr_##fn = (typeof(ptr_##fn))           \
+        ptr_##fn = (typeof(ptr_##fn)) (void *)  \
             GetProcAddress(hmodule,#fn);        \
     } while (0)
 
@@ -820,20 +758,21 @@ static inline boolean local_thread_stack_address_p(os_vm_address_t address)
  */
 
 os_vm_address_t
-os_validate(int movable, os_vm_address_t addr, os_vm_size_t len)
+os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len)
 {
     MEMORY_BASIC_INFORMATION mem_info;
 
     if (!addr) {
         /* the simple case first */
+        int protection = attributes & IS_GUARD_PAGE ? PAGE_NOACCESS : PAGE_EXECUTE_READWRITE;
         return
-            AVERLAX(VirtualAlloc(addr, len, MEM_RESERVE|MEM_COMMIT, PAGE_EXECUTE_READWRITE));
+            AVERLAX(VirtualAlloc(addr, len, MEM_RESERVE|MEM_COMMIT, protection));
     }
 
     if (!AVERLAX(VirtualQuery(addr, &mem_info, sizeof mem_info)))
         return 0;
 
-    if ((mem_info.State == MEM_RESERVE) && (mem_info.RegionSize >=len)) {
+    if ((mem_info.State == MEM_RESERVE) && (mem_info.RegionSize >= len)) {
         /* It would be correct to return here. However, support for Wine
          * is beneficial, and Wine has a strange behavior in this
          * department. It reports all memory below KERNEL32.DLL as
@@ -845,23 +784,36 @@ os_validate(int movable, os_vm_address_t addr, os_vm_size_t len)
          * actually free.
          */
         VirtualAlloc(addr, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE);
-        /* If it is wine, the second call has succeded, and now the region
+        /* If it is wine, the second call has succeeded, and now the region
          * is really reserved. */
         return addr;
     }
 
+    DWORD mode;
     if (mem_info.State == MEM_RESERVE) {
         fprintf(stderr, "validation of reserved space too short.\n");
         fflush(stderr);
         /* Oddly, we do not treat this assertion as fatal; hence also the
          * provision for MEM_RESERVE in the following code, I suppose: */
+        mode = MEM_COMMIT;
+    } else {
+        mode = MEM_RESERVE;
     }
 
-    os_vm_address_t actual;
+    os_vm_address_t actual = VirtualAlloc(addr, len, mode, PAGE_EXECUTE_READWRITE);
+
+    if (!actual) {
+        if (!(attributes & MOVABLE)) {
+            fprintf(stderr,
+                    "VirtualAlloc: wanted %lu bytes at %p, actually mapped at %p\n",
+                    (unsigned long) len, addr, actual);
+            fflush(stderr);
+            return 0;
+        }
+
+        return AVERLAX(VirtualAlloc(NULL, len, mode, PAGE_EXECUTE_READWRITE));
+    }
 
-    if (!AVERLAX(actual = VirtualAlloc(addr, len, (mem_info.State == MEM_RESERVE)?
-                                       MEM_COMMIT: MEM_RESERVE, PAGE_EXECUTE_READWRITE)))
-        return 0;
     return actual;
 }
 
@@ -925,7 +877,7 @@ os_invalidate_free_by_any_address(os_vm_address_t addr,
 
 /* os_validate doesn't commit, i.e. doesn't actually "validate" in the
  * sense that we could start using the space afterwards.  Usually it's
- * os_map or Lisp code that will run into that, in which case we recommit
+ * load_core_bytes or Lisp code that will run into that, in which case we recommit
  * elsewhere in this file.  For cases where C wants to write into newly
  * os_validate()d memory, it needs to commit it explicitly first:
  */
@@ -937,7 +889,7 @@ os_validate_recommit(os_vm_address_t addr, os_vm_size_t len)
 }
 
 /*
- * os_map() is called to map a chunk of the core file into memory.
+ * load_core_bytes() is called to load a chunk of the core file into memory.
  *
  * Unfortunately, Windows semantics completely screws this up, so
  * we just add backing store from the swapfile to where the chunk
@@ -947,18 +899,28 @@ os_validate_recommit(os_vm_address_t addr, os_vm_size_t len)
  * thing to maintain).
  */
 
-void os_map(int fd, int offset, os_vm_address_t addr, os_vm_size_t len)
+void* load_core_bytes(int fd, os_vm_offset_t offset, os_vm_address_t addr, os_vm_size_t len)
 {
-    os_vm_size_t count;
 
     AVER(VirtualAlloc(addr, len, MEM_COMMIT, PAGE_EXECUTE_READWRITE)||
          VirtualAlloc(addr, len, MEM_RESERVE|MEM_COMMIT,
                       PAGE_EXECUTE_READWRITE));
-
+#ifdef LISP_FEATURE_64_BIT
+    CRT_AVER_NONNEGATIVE(_lseeki64(fd, offset, SEEK_SET));
+#else
     CRT_AVER_NONNEGATIVE(lseek(fd, offset, SEEK_SET));
+#endif
+    size_t count;
 
-    count = read(fd, addr, len);
-    CRT_AVER( count == len );
+    while (len) {
+        unsigned to_read = len > INT_MAX ? INT_MAX : len;
+        count = read(fd, addr, to_read);
+        addr += count;
+        len -= count;
+        CRT_AVER(count == to_read);
+    }
+
+    return (void*)0;
 }
 static DWORD os_protect_modes[8] = {
     PAGE_NOACCESS,
@@ -1372,6 +1334,7 @@ handle_exception(EXCEPTION_RECORD *exception_record,
     DWORD code = exception_record->ExceptionCode;
 
     if(code == 0x20474343 || /* GCC */
+       code == 0x406D1388 || /* MS_VC_EXCEPTION */
        code == 0xE06D7363 || /* Emsc */
        code == 0xE0434352)   /* ECCR */
         /* Do not handle G++, VC++ and .NET exceptions */
@@ -1388,7 +1351,7 @@ handle_exception(EXCEPTION_RECORD *exception_record,
     context.sigmask = self ? self->os_thread->blocked_signal_set : 0;
 #endif
 
-    os_context_register_t oldbp = NULL;
+    os_context_register_t oldbp = 0;
     if (self) {
         oldbp = self ? self->carried_base_pointer : 0;
         self->carried_base_pointer
@@ -1530,7 +1493,7 @@ char *dirname(char *path)
     int i;
 
     if (pathlen >= sizeof(buf)) {
-        lose("Pathname too long in dirname.\n");
+        lose("Pathname too long in dirname.");
         return NULL;
     }
 
@@ -1549,19 +1512,32 @@ char *dirname(char *path)
 int
 socket_input_available(HANDLE socket)
 {
-    unsigned long count = 0, count_size = 0;
+    int count = 0;
     int wsaErrno = GetLastError();
-    int err = WSAIoctl((SOCKET)socket, FIONREAD, NULL, 0,
-                       &count, sizeof(count), &count_size, NULL, NULL);
+    TIMEVAL timeout = {0, 0};
+    fd_set readfds, errfds;
 
-    int ret;
+    FD_ZERO(&readfds);
+    FD_ZERO(&errfds);
+    FD_SET(socket, &readfds);
+    FD_SET(socket, &errfds);
 
-    if (err == 0) {
-        ret = (count > 0) ? 1 : 2;
-    } else
-        ret = 0;
+    count = select(0, &readfds, NULL, &errfds, &timeout);
     SetLastError(wsaErrno);
-    return ret;
+
+    if (count == SOCKET_ERROR)
+        /* This is an error in select() itself, not with the socket. */
+        return 0;
+    else if (count == 0)
+        return 2;
+    /* True if there is data, or the socket has been closed (i.e. a read
+     * would retrieve an EOF error) */
+    else if (FD_ISSET(socket, &readfds))
+        return 1;
+    else
+        /* This is an error in the socket. The count was positive, and
+         * it wasn't in the read set, so it must be in the error set. */
+        return 0;
 }
 
 #ifdef LISP_FEATURE_SB_THREAD
@@ -1689,7 +1665,7 @@ struct {
     boolean in_progress;
 } ttyinput = {.lock = PTHREAD_MUTEX_INITIALIZER};
 
-static void*
+static unsigned long
 tty_read_line_server()
 {
     pthread_mutex_lock(&ttyinput.lock);
@@ -1736,7 +1712,10 @@ tty_maybe_initialize_unlocked(HANDLE handle)
         }
         pthread_cond_init(&ttyinput.cond_has_data,NULL);
         pthread_cond_init(&ttyinput.cond_has_client,NULL);
-        pthread_create(&ttyinput.thread,NULL,tty_read_line_server,NULL);
+        ttyinput.thread = CreateThread(NULL,
+                                       0, // stack size = default
+                                       tty_read_line_server, 0,
+                                       0, 0);
         ttyinput.initialized = 1;
     }
     return 1;
@@ -2100,8 +2079,7 @@ void scratch(void)
     CryptGenRandom(0, 0, 0);
 }
 
-char *
-os_get_runtime_executable_path(int __attribute__((__unused__)) external)
+char *os_get_runtime_executable_path()
 {
     char path[MAX_PATH + 1];
     DWORD bufsize = sizeof(path);
